@@ -1,10 +1,10 @@
 /***************************************************************************
- *   Copyright (C) 2005,2006 by Jonathan Duddington                        *
- *   jonsd@users.sourceforge.net                                           *
+ *   Copyright (C) 2005 to 2007 by Jonathan Duddington                     *
+ *   email: jonsd@users.sourceforge.net                                    *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
- *   the Free Software Foundation; either version 2 of the License, or     *
+ *   the Free Software Foundation; either version 3 of the License, or     *
  *   (at your option) any later version.                                   *
  *                                                                         *
  *   This program is distributed in the hope that it will be useful,       *
@@ -13,10 +13,10 @@
  *   GNU General Public License for more details.                          *
  *                                                                         *
  *   You should have received a copy of the GNU General Public License     *
- *   along with this program; if not, write to the                         *
- *   Free Software Foundation, Inc.,                                       *
- *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
+ *   along with this program; if not, write see:                           *
+ *               <http://www.gnu.org/licenses/>.                           *
  ***************************************************************************/
+
 #include "StdAfx.h"
 
 #include <stdio.h>
@@ -31,7 +31,6 @@
 #include "synthesize.h"
 #include "translate.h"
 
-#define OPT_PH_COMMON      // sort rules by phoneme string, common phoneme string output only once
 //#define OPT_FORMAT         // format the text and write formatted copy to Log file 
 //#define OUTPUT_FORMAT
 
@@ -162,6 +161,7 @@ int compile_line(char *linebuf, char *dict_line, int *hash)
 	int  ix;
 	int  step;
 	int  n_flag_codes = 0;
+	int  flag_offset;
 	int  length;
 	int  multiple_words = 0;
 	char *multiple_string = NULL;
@@ -190,7 +190,17 @@ int compile_line(char *linebuf, char *dict_line, int *hash)
 	
 		if((c == '?') && (step==0))
 		{
+			// conditional rule, allow only if the numbered condition is set for the voice
+			flag_offset = 100;
+
 			p++;
+			if(*p == '!')
+			{
+				// allow only if the numbered condition is NOT set
+				flag_offset = 132;
+				p++;
+			}
+
 			ix = 0;
 			if(isdigit(*p))
 			{
@@ -202,11 +212,11 @@ int compile_line(char *linebuf, char *dict_line, int *hash)
 				ix = ix*10 + (*p-'0');
 				p++;
 			}
-			flag_codes[n_flag_codes++] = ix+100;
+			flag_codes[n_flag_codes++] = ix + flag_offset;
 			c = *p;
 		}
 		
-		if((c == '$') && (step != 1))
+		if((c == '$') && isalnum(p[1]))
 		{
 			/* read keyword parameter */
 			mnemptr = p;
@@ -651,13 +661,24 @@ void copy_rule_string(char *string, int &state)
 				case '#':
 					c = RULE_DEL_FWD;
 					break;
-
+				case '!':
+					c = RULE_CAPITAL;
+					break;
+				case 'T':
+					c = RULE_ALT1;
+					break;
+				case 'W':
+					c = RULE_SPELLING;
+					break;
+				case 'X':
+					c = RULE_NOVOWELS;
+					break;
 				case 'L':
 					// expect two digits
 					c = *p++ - '0';
 					value = *p++ - '0';
 					c = c * 10 + value;
-					if((value < 0) || (value > 9) || (c <= 0) || (c >= N_LETTER_TYPES))
+					if((value < 0) || (value > 9) || (c <= 0) || (c >= N_LETTER_GROUPS))
 					{
 						c = 0;
 						fprintf(f_log,"%5d: Expected 2 digits after 'L'",linenum);
@@ -668,11 +689,11 @@ void copy_rule_string(char *string, int &state)
 					{
 						// pre-rule, put the group number before the RULE_LETTERGP command
 						output[ix++] = c;
-						c = RULE_LETTERGP;
+						c = RULE_LETTERGP2;
 					}
 					else
 					{
-						output[ix++] = RULE_LETTERGP;
+						output[ix++] = RULE_LETTERGP2;
 					}
 					break;
 
@@ -681,7 +702,7 @@ void copy_rule_string(char *string, int &state)
 				case '$':   // obsolete, replaced by S
 				case 'S':
 					output[ix++] = RULE_ENDING;
-					value = 0x80;
+					value = 0;
 					while(!isspace2(c = *p++) && (c != 0))
 					{
 						switch(c)
@@ -712,14 +733,14 @@ void copy_rule_string(char *string, int &state)
 							break;
 						default:
 							if(isdigit(c))
-								value = (c - '0') + 0x80;
+								value = (value*10) + (c - '0');
 							break;
 						}
 					}
 					p--;
 					output[ix++] = sxflags >> 16;
 					output[ix++] = sxflags >> 8;
-					c = value;
+					c = value | 0x80;
 					break;
 				}
 			}
@@ -849,7 +870,17 @@ char *compile_rule(char *input)
 	if(rule_cond[0] != 0)
 	{
 		ix = -1;
-		ix = atoi(rule_cond);
+		if(rule_cond[0] == '!')
+		{
+			// allow the rule only if the condition number is NOT set for the voice
+			ix = atoi(&rule_cond[1]) + 32;
+		}
+		else
+		{
+			// allow the rule only if the condition number is set for the voice
+			ix = atoi(rule_cond);
+		}
+
 		if((ix > 0) && (ix < 255))
 		{
 			output[len++] = RULE_CONDITION;
@@ -891,7 +922,8 @@ static int __cdecl string_sorter(char **a, char **b)
 	pa += (strlen(pa)+1);
 	pb += (strlen(pb)+1);
    return(strcmp(pa,pb));
-}   /* end of strcmp2 */
+}   /* end of string_sorter */
+
 
 static int __cdecl rgroup_sorter(RGROUP *a, RGROUP *b)
 {//===================================================
@@ -1009,7 +1041,7 @@ void print_rule_group(FILE *f_out, int n_rules, char **rules, char *name)
 #endif
 
 
-
+//#define LIST_GROUP_INFO
 void output_rule_group(FILE *f_out, int n_rules, char **rules, char *name)
 {//=======================================================================
 	int ix;
@@ -1020,17 +1052,18 @@ void output_rule_group(FILE *f_out, int n_rules, char **rules, char *name)
 	char *p2, *p3;
 	const char *common;
 
-//fprintf(f_log,"Group %2s n=%2d at 0x%x\n",name,n_rules,ftell(f_out));
+	short nextchar_count[256];
+	memset(nextchar_count,0,sizeof(nextchar_count));
+
 	len_name = strlen(name);
 
 #ifdef OUTPUT_FORMAT
 	print_rule_group(f_log,n_rules,rules,name);
 #endif
 
+	// sort the rules in this group by their phoneme string
 	common = "";
-#ifdef OPT_PH_COMMON
 	qsort((void *)rules,n_rules,sizeof(char *),(int (__cdecl *)(const void *,const void *))string_sorter);
-#endif
 
 	if(strcmp(name,"9")==0)
 		len_name = 0;    //  don't remove characters from numeric match strings
@@ -1042,8 +1075,8 @@ void output_rule_group(FILE *f_out, int n_rules, char **rules, char *name)
 		p3 = &p[len1];
 		p2 = p3 + len_name;        // remove group name from start of match string
 		len2 = strlen(p2);
-//fprintf(f_log,"%4x  %2d  len1=%2d  %2x %2x %2x %2x len2=%2d  %2x %2x %2x %2x %2x %2x %2x\n",
-//	ftell(f_out),ix,len1,p[0],p[1],p[2],p[3],len2,p3[0],p3[1],p3[2],p3[3],p3[4],p3[5],p3[6]);
+
+		nextchar_count[(unsigned char)(p2[0])]++;   // the next byte after the group name
 
 		if((common[0] != 0) && (strcmp(p,common)==0))
 		{
@@ -1052,24 +1085,70 @@ void output_rule_group(FILE *f_out, int n_rules, char **rules, char *name)
 		}
 		else
 		{
-#ifdef OPT_PH_COMMON
 			if((ix < n_rules-1) && (strcmp(p,rules[ix+1])==0))
 			{
 				common = rules[ix];   // phoneme string is same as next, set as common
 				fputc(RULE_PH_COMMON,f_out);
 			}
-#endif
+
 			fwrite(p2,len2,1,f_out);
 			fputc(RULE_PHONEMES,f_out);
 			fwrite(p,len1,1,f_out);
 		}
 	}
+
+#ifdef LIST_GROUP_INFO
+	for(ix=32; ix<256; ix++)
+	{
+		if(nextchar_count[ix] > 30)
+			printf("Group %s   %c  %d\n",name,ix,nextchar_count[ix]);
+	}
+#endif
 }  //  end of output_rule_group
 
 
 
-int compile_dictrules(FILE *f_in, FILE *f_out, char *fname_temp)
-{//=============================================================
+static int compile_lettergroup(char *input, FILE *f_out)
+{//=====================================================
+	char *p;
+	int group;
+
+	p = input;
+	if(!isdigit(p[0]) || !isdigit(p[1]))
+	{
+		return(1);
+	}
+
+	group = atoi(&p[1]);
+	if(group >= N_LETTER_GROUPS)
+		return(1);
+
+	while(!isspace2(*p)) p++;
+
+	fputc(RULE_GROUP_START,f_out);
+	fputc(RULE_LETTERGP2,f_out);
+	fputc(group + 'A', f_out);
+
+	for(;;)
+	{
+		while(isspace2(*p)) p++;
+		if(*p == 0)
+			break;
+		
+		while((*p & 0xff) > ' ')
+		{
+			fputc(*p++, f_out);
+		}
+		fputc(0,f_out);
+	}
+	fputc(RULE_GROUP_END,f_out);
+
+	return(0);
+}
+
+
+static int compile_dictrules(FILE *f_in, FILE *f_out, char *fname_temp)
+{//====================================================================
 	char *prule;
 	unsigned char *p;
 	int ix;
@@ -1100,6 +1179,16 @@ int compile_dictrules(FILE *f_in, FILE *f_out, char *fname_temp)
 		buf = fgets(buf1,sizeof(buf1),f_in);
 		if((buf != NULL) && (buf[0] == '\r')) buf++;  // ignore extra \r in \r\n 
 
+		if((buf != NULL) && (memcmp(buf,".L",2)==0))
+		{
+			if(compile_lettergroup(&buf[2], f_out) != 0)
+			{
+				fprintf(f_log,"%5d: Bad lettergroup\n",linenum);
+				error_count++;
+			}
+			continue;
+		}
+
 		if((buf == NULL) || (memcmp(buf,".group",6)==0))
 		{
 			// next .group or end of file, write out the previous group
@@ -1212,9 +1301,9 @@ int CompileDictionary(const char *dsource, const char *dict_name, FILE *log, cha
 	FILE *f_out;
 	int offset_rules=0;
 	int value;
-	char fname_buf[130];
-	char fname_temp[130];
-	char path[80];
+	char fname_buf[sizeof(path_home)+15];
+	char fname_temp[sizeof(path_home)+15];
+	char path[sizeof(path_home)+40];       // path_dsource+20
 
 	error_count = 0;
 
