@@ -1,10 +1,10 @@
 /***************************************************************************
- *   Copyright (C) 2005,2006 by Jonathan Duddington                        *
- *   jonsd@users.sourceforge.net                                           *
+ *   Copyright (C) 2005 to 2007 by Jonathan Duddington                     *
+ *   email: jonsd@users.sourceforge.net                                    *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
- *   the Free Software Foundation; either version 2 of the License, or     *
+ *   the Free Software Foundation; either version 3 of the License, or     *
  *   (at your option) any later version.                                   *
  *                                                                         *
  *   This program is distributed in the hope that it will be useful,       *
@@ -13,10 +13,10 @@
  *   GNU General Public License for more details.                          *
  *                                                                         *
  *   You should have received a copy of the GNU General Public License     *
- *   along with this program; if not, write to the                         *
- *   Free Software Foundation, Inc.,                                       *
- *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
+ *   along with this program; if not, write see:                           *
+ *               <http://www.gnu.org/licenses/>.                           *
  ***************************************************************************/
+
 #include "StdAfx.h"
 
 #define LOG_TRANSLATE
@@ -42,8 +42,8 @@ char dictionary_name[40];
 extern MNEM_TAB mnem_flags[];
 
 // accented characters which indicate (in some languages) the start of a separate syllable
-//static const short diereses_list[7] = {L'ä',L'ë',L'ï',L'ö',L'ü',L'ÿ',0};
-static const short diereses_list[7] = {0xe4,0xeb,0xef,0xf6,0xfc,0xff,0};
+//static const unsigned short diereses_list[7] = {L'ä',L'ë',L'ï',L'ö',L'ü',L'ÿ',0};
+static const unsigned short diereses_list[7] = {0xe4,0xeb,0xef,0xf6,0xfc,0xff,0};
 
 // convert characters to an approximate 7 bit ascii equivalent
 // used for checking for vowels
@@ -127,7 +127,7 @@ int Translator::LoadDictionary(const char *name, int no_error)
 	int length;
 	FILE *f;
 	unsigned int size;
-	char fname[130];
+	char fname[sizeof(path_home)+20];
 
 	strcpy(dictionary_name,name);   // currently loaded dictionary name
 
@@ -213,6 +213,7 @@ void Translator::InitGroups(void)
 		groups2_count[ix]=0;
 		groups2_start[ix]=255;  // indicates "not set"
 	}
+	memset(letterGroups,0,sizeof(letterGroups));
 
 	p = data_dictrules;
 	while(*p != 0)
@@ -223,30 +224,44 @@ void Translator::InitGroups(void)
 			break;
 		}
 		p++;
-		len = strlen(p);
-		p_name = p;
-		c = p_name[0];
-		
-		p += (len+1);
-		if(len == 1)
-		{
-			groups1[c] = p;
-		}
-		else
-		if(len == 0)
-		{
-			groups1[0] = p;
-		}
-		else
-		{
-			if(groups2_start[c] == 255)
-				groups2_start[c] = n_groups2;
 
-			groups2_count[c]++;
-			groups2[n_groups2] = p;
-			c2 = p_name[1];
-			groups2_name[n_groups2++] = (c + (c2 << 8));
+		if(p[0] == RULE_LETTERGP2)
+		{
+			ix = p[1] - 'A';
+			p += 2;
+			if((ix >= 0) && (ix < N_LETTER_GROUPS))
+			{
+				letterGroups[ix] = p;
+			}
 		}
+		else
+		{
+			len = strlen(p);
+			p_name = p;
+			c = p_name[0];
+			
+			p += (len+1);
+			if(len == 1)
+			{
+				groups1[c] = p;
+			}
+			else
+			if(len == 0)
+			{
+				groups1[0] = p;
+			}
+			else
+			{
+				if(groups2_start[c] == 255)
+					groups2_start[c] = n_groups2;
+	
+				groups2_count[c]++;
+				groups2[n_groups2] = p;
+				c2 = p_name[1];
+				groups2_name[n_groups2++] = (c + (c2 << 8));
+			}
+		}
+
 		// skip over all the rules in this group
 		rule_count = 0;
 		while(*p != RULE_GROUP_END)
@@ -589,6 +604,31 @@ return(0);
 
 
 
+int Translator::IsLetterGroup(char *word, int group)
+{//=================================================
+	// match the word against a list of utf-8 strings
+	char *p;
+	char *w;
+
+	p = letterGroups[group];
+
+	while(*p != 0)
+	{
+		w = word;
+		while(*p == *w)
+		{
+			*w++;
+			*p++;
+		}
+		if(*p == 0)
+			return(w-word);   // matched a complete string
+
+		while(*p++ != 0);  // skip to end of string
+	}
+	return(0);
+}
+
+
 int Translator::IsLetter(int letter, int group)
 {//============================================
 	if(letter_groups[group] != NULL)
@@ -626,9 +666,24 @@ int Translator::IsVowel(int letter)
 
 void SetLetterVowel(Translator *tr, int c)
 {//=======================================
-	tr->letter_bits[c] = 0x01;   // group 0 only
+	tr->letter_bits[c] = (tr->letter_bits[c] & 0x40) | 0x81;  // keep value for group 6 (front vowels e,i,y)
 }
 
+void ResetLetterBits(Translator *tr, int groups)
+{//=============================================
+// Clear all the specified groups
+	unsigned int ix;
+	unsigned int mask;
+
+	mask = ~groups;
+
+	for(ix=0; ix<sizeof(tr->letter_bits); ix++)
+	{
+		tr->letter_bits[ix] &= mask;
+	}
+}
+
+
 void SetLetterBits(Translator *tr, int group, const char *string)
 {//==============================================================
 	int bits;
@@ -660,8 +715,8 @@ void SetLetterBitsRange(Translator *tr, int group, int first, int last)
 
 
 
-int GetVowelStress(unsigned char *phonemes, unsigned char *vowel_stress, int &vowel_count, int &stressed_syllable)
-{//===============================================================================================================
+static int GetVowelStress(Translator *tr, unsigned char *phonemes, unsigned char *vowel_stress, int &vowel_count, int &stressed_syllable)
+{//======================================================================================================================================
 
 	unsigned char phcode;
 	PHONEME_TAB *ph;
@@ -724,7 +779,7 @@ int GetVowelStress(unsigned char *phonemes, unsigned char *vowel_stress, int &vo
 			continue;
 		}
 
-		if(ph->type == phVOWEL)
+		if((ph->type == phVOWEL) && !(ph->phflags & phNONSYLLABIC))
 		{
 			vowel_stress[count] = (char)stress;
 			if((stress >= 4) && (stress >= max_stress))
@@ -770,7 +825,12 @@ int GetVowelStress(unsigned char *phonemes, unsigned char *vowel_stress, int &vo
 		for(ix=1; ix<count; ix++)
 		{
 			if(vowel_stress[ix] == 4)
-				vowel_stress[ix] = 0;
+			{
+				if(tr->langopts.stress_flags & 0x20000)
+					vowel_stress[ix] = 0;
+				else
+					vowel_stress[ix] = 3;
+			}
 
 			if(vowel_stress[ix] == 5)
 			{
@@ -792,8 +852,8 @@ static char stress_phonemes[] = {phonSTRESS_U, phonSTRESS_D, phonSTRESS_2, phonS
 		phonSTRESS_P, phonSTRESS_TONIC, phonSTRESS_TONIC};
 
 
-void ChangeWordStress(char *word, int new_stress)
-{//==============================================
+void ChangeWordStress(Translator *tr, char *word, int new_stress)
+{//==============================================================
 	int ix;
 	unsigned char *p;
 	int  max_stress;
@@ -803,7 +863,7 @@ void ChangeWordStress(char *word, int new_stress)
 	unsigned char vowel_stress[N_WORD_PHONEMES/2];
 
 	strcpy((char *)phonetic,word);
-	max_stress = GetVowelStress(phonetic,vowel_stress,vowel_count,stressed_syllable);
+	max_stress = GetVowelStress(tr, phonetic,vowel_stress,vowel_count,stressed_syllable);
 
 	if(new_stress >= 4)
 	{
@@ -832,7 +892,7 @@ void ChangeWordStress(char *word, int new_stress)
 	p = phonetic;
 	while(*p != 0)
 	{
-		if(phoneme_tab[*p]->type == phVOWEL)
+		if((phoneme_tab[*p]->type == phVOWEL) && !(phoneme_tab[*p]->phflags & phNONSYLLABIC))
 		{
 			if(vowel_stress[ix] != 0)
 				*word++ = stress_phonemes[vowel_stress[ix]];
@@ -876,6 +936,8 @@ void Translator::SetWordStress(char *output, unsigned int dictionary_flags, int
 	int final_ph;
 	int mnem;
 	int post_tonic;
+	int opt_length;
+	int done;
 
 	unsigned char vowel_stress[N_WORD_PHONEMES/2];
 	char syllable_type[N_WORD_PHONEMES/2];
@@ -914,13 +976,13 @@ void Translator::SetWordStress(char *output, unsigned int dictionary_flags, int
 		unstressed_word = 1;
 	}
 
-	max_stress = GetVowelStress(phonetic,vowel_stress,vowel_count,stressed_syllable);
+	max_stress = GetVowelStress(this,phonetic,vowel_stress,vowel_count,stressed_syllable);
 
 	// heavy or light syllables
 	ix = 1;
 	for(p = phonetic; *p != 0; p++)
 	{
-		if(phoneme_tab[p[0]]->type == phVOWEL)
+		if((phoneme_tab[p[0]]->type == phVOWEL) && !(phoneme_tab[p[0]]->phflags & phNONSYLLABIC))
 		{
 			int weight = 0;
 			int lengthened = 0;
@@ -1020,6 +1082,21 @@ void Translator::SetWordStress(char *output, unsigned int dictionary_flags, int
 		}
 		break;
 
+	case 4:   // stress on antipenultimate vowel
+		if(stressed_syllable == 0)
+		{
+			stressed_syllable = vowel_count - 3;
+			if(stressed_syllable < 1)
+				stressed_syllable = 1;
+
+			if(max_stress == 0)
+			{
+				vowel_stress[stressed_syllable] = 4;
+			}
+			max_stress = 4;
+		}
+		break;
+
 	case 5:
 		// LANG=Russian
 		if(stressed_syllable == 0)
@@ -1096,15 +1173,26 @@ void Translator::SetWordStress(char *output, unsigned int dictionary_flags, int
 		stress = 3;
 
 
-	if((langopts.stress_flags & 0x1000) && (vowel_stress[1] == 0))
+	if((langopts.stress_flags & 0x1000) && (vowel_count == 2))
 	{
-		if((vowel_count == 3) && (vowel_stress[2] >= 4))
+		// Two syllable word, if one syllable has primary stress, then give the other secondary stress
+		if(vowel_stress[1] == 4)
+			vowel_stress[2] = 3;
+		if(vowel_stress[2] == 4)
+			vowel_stress[1] = 3;
+	}
+#if deleted
+	if((langopts.stress_flags & 0x2000) && (vowel_stress[1] == 0))
+	{
+		// If there is only one syllable before the primary stress, give it a secondary stress
+		if((vowel_count > 2) && (vowel_stress[2] >= 4))
 		{
-			// 2-syllable word, put a secondary stress on the initial syllable
 			vowel_stress[1] = 3;
 		}
 	}
+#endif
 
+	done = 0;
 	for(v=1; v<vowel_count; v++)
 	{
 		if(vowel_stress[v] == 0)
@@ -1114,6 +1202,13 @@ void Translator::SetWordStress(char *output, unsigned int dictionary_flags, int
 				// flag: don't give secondary stress to final vowel
 			}
 			else
+			if((langopts.stress_flags & 0x8000) && (done == 0))
+			{
+				vowel_stress[v] = (char)stress;
+				done =1;
+				stress = 3;  /* use secondary stress for remaining syllables */
+			}
+			else
 			if((vowel_stress[v-1] <= 1) && (vowel_stress[v+1] <= 1))
 			{
 				/* trochaic: give stress to vowel surrounded by unstressed vowels */
@@ -1130,6 +1225,7 @@ void Translator::SetWordStress(char *output, unsigned int dictionary_flags, int
 // should start with secondary stress on the first syllable, or should it count back from
 // the primary stress and put secondary stress on alternate syllables?
 				vowel_stress[v] = (char)stress;
+				done =1;
 				stress = 3;  /* use secondary stress for remaining syllables */
 			}
 		}
@@ -1143,19 +1239,20 @@ void Translator::SetWordStress(char *output, unsigned int dictionary_flags, int
 			tonic = langopts.unstressed_wd2;   /* more than one syllable, used secondary stress as the main stress */
 	}
 
+	max_stress = 0;
+	max_stress_posn = 0;
+	for(v=1; v<vowel_count; v++)
+	{
+		if(vowel_stress[v] >= max_stress)
+		{
+			max_stress = vowel_stress[v];
+			max_stress_posn = v;
+		}
+	}
+
 	if(tonic >= 0)
 	{
 		/* find position of highest stress, and replace it by 'tonic' */
-		max_stress = 0;
-		max_stress_posn = 0;
-		for(v=1; v<vowel_count; v++)
-		{
-			if(vowel_stress[v] >= max_stress)
-			{
-				max_stress = vowel_stress[v];
-				max_stress_posn = v;
-			}
-		}
 
 		/* don't disturb an explicitly set stress by 'unstress-at-end' flag */
 		if((tonic > max_stress) || (max_stress <= 4))
@@ -1217,7 +1314,7 @@ void Translator::SetWordStress(char *output, unsigned int dictionary_flags, int
 			prev_last_stress = 0;
 		}
 		else
-		if((ph->type == phVOWEL) || (*p == phonSYLLABIC))
+		if(((ph->type == phVOWEL) && !(ph->phflags & phNONSYLLABIC)) || (*p == phonSYLLABIC))
 		{
 			// a vowel, or a consonant followed by a syllabic consonant marker
 
@@ -1249,14 +1346,18 @@ void Translator::SetWordStress(char *output, unsigned int dictionary_flags, int
 				else
 				{
 					// unstressed syllable within a word
-					v_stress = 1;      /* change from 0 (unstressed) to 1 (diminished stress) */
+					if((vowel_stress[v-1] != 1) || ((langopts.stress_flags & 0x10000) == 0))
+					{
+						v_stress = 1;      /* change from 0 (unstressed) to 1 (diminished stress) */
+						vowel_stress[v] = v_stress;
+					}
 				}
 			}
 
 			if(v_stress > 0)
 				*output++ = stress_phonemes[v_stress];  // mark stress of all vowels except 0 (unstressed)
 
-			if((ph->reduce_to != 0) && !(dictionary_flags & FLAG_FOUND))
+			while((ph->reduce_to != 0) && (((dictionary_flags & FLAG_FOUND)==0) || (langopts.param[LOPT_REDUCE] & 1)))
 			{
 				// this vowel can be reduced to another if the stress is below a specified value
 				int reduce = 0;
@@ -1282,14 +1383,16 @@ void Translator::SetWordStress(char *output, unsigned int dictionary_flags, int
 				}
 
 				if(max_stress == 1) max_stress = 0;
-				if(unstressed_word && langopts.param[LOPT_KEEP_UNSTR_VOWEL] && (v_stress >= max_stress))
+				if(unstressed_word && (langopts.param[LOPT_REDUCE] & 0x2) && (v_stress >= max_stress))
 				{
+					// don't reduce the most stressed syllable in an unstressed word
 					reduce = 0;
 				}
 
 				if(reduce)
 				{
 					phcode = ph->reduce_to;
+					ph = phoneme_tab[phcode];
 #ifdef deleted
 					if(*p == phonLENGTHEN)
 					{
@@ -1298,22 +1401,40 @@ void Translator::SetWordStress(char *output, unsigned int dictionary_flags, int
 					}
 #endif
 				}
-			}
-
-			if(langopts.param[LOPT_IT_LENGTHEN] && (*p == phonLENGTHEN))
-			{
-				// remove lengthen indicator from non-stressed syllables
-				if(v_stress < 4)
-					p++;
 				else
-				if((langopts.param[LOPT_IT_LENGTHEN]==2) && (v != (vowel_count - 2)))
-					p++;    // LANG=Italian, remove lengthen indicator from non-penultimate syllables
+					break;
 			}
 
 			if(vowel_stress[v] > max_stress)
 			{
 				max_stress = vowel_stress[v];
 			}
+
+			if((*p == phonLENGTHEN) && ((opt_length = langopts.param[LOPT_IT_LENGTHEN]) != 0))
+			{
+				// remove lengthen indicator from non-stressed syllables
+				int shorten=0;
+
+				if(opt_length & 0x10)
+				{
+					// only allow lengthen indicator on the highest stress syllable in the word
+					if(v != max_stress_posn)
+						shorten = 1;
+				}
+				else
+				if(v_stress < 4)
+				{
+					// only allow lengthen indicator if stress >= 4.
+					shorten = 1;
+				}
+
+				if(((opt_length & 0xf)==2) && (v != (vowel_count - 2)))
+					shorten = 1;    // LANG=Italian, remove lengthen indicator from non-penultimate syllables
+
+				if(shorten)
+					p++;
+			}
+
 			v++;
 		}
 
@@ -1345,15 +1466,18 @@ char *Translator::DecodeRule(const char *group, char *rule)
 	int  match_type;
 	int  finished=0;
 	int  value;
+	int  flags;
+	int  suffix_char;
 	int  condition_num=0;
    char buf[60];
    char buf_pre[60];
-	char suffix[12];
+	char suffix[20];
 	static char output[60];
 
 	static char symbols[] = {' ',' ',' ',' ',' ',' ',' ',' ',' ',
-			'@','&','%','+','#','S','D','Z','A','B','C','H','F','G','Y','N','K','V','L'};
+			'@','&','%','+','#','S','D','Z','A','L',' ',' ',' ',' ',' ','N','K','V',' ','T','X','?','W'};
 
+	static char symbols_lg[] = {'A','B','C','H','F','G','Y'};
 
 	match_type = 0;
    buf_pre[0] = 0;
@@ -1394,36 +1518,42 @@ char *Translator::DecodeRule(const char *group, char *rule)
 		
 		if(rb == RULE_ENDING)
 		{
-			if(rule[0] & (SUFX_P>>8))
-				sprintf(suffix,"P%d",rule[1] & 0x7f);
-			else
-				sprintf(suffix,"$%d(%x)",rule[1] & 0x7f,rule[0] & 0x7f);
-			rule += 2;
+			static char *flag_chars = "ei vtfq t";
+			flags = ((rule[0] & 0x7f)<< 8) + (rule[1] & 0x7f);
+			suffix_char = 'S';
+			if(flags & (SUFX_P >> 8))
+				suffix_char = 'P';
+			sprintf(suffix,"%c%d",suffix_char,rule[2] & 0x7f);
+			rule += 3;
+			for(ix=0;ix<9;ix++)
+			{
+				if(flags & 1)
+					sprintf(&suffix[strlen(suffix)],"%c",flag_chars[ix]);
+				flags = (flags >> 1);
+			}
 			strcpy(p,suffix);
 			p += strlen(suffix);
 			c = ' ';
 		}
 		else
 		if(rb == RULE_LETTERGP)
+		{
+			c = symbols_lg[*rule++ - 'A'];
+		}
+		else
+		if(rb == RULE_LETTERGP2)
 		{
 			value = *rule++ - 'A';
-			if(value >= 8)
-			{
-				p[0] = 'L';
-				p[1] = (value / 10) + '0';
-				c = (value % 10) + '0';
+			p[0] = 'L';
+			p[1] = (value / 10) + '0';
+			c = (value % 10) + '0';
 
-				if(match_type == RULE_PRE)
-				{
-					p[0] = c;
-					c = 'L';
-				}
-				p+=2;
-			}
-			else
+			if(match_type == RULE_PRE)
 			{
-				c = symbols[value + RULE_LETTER_GROUPS];
+				p[0] = c;
+				c = 'L';
 			}
+			p+=2;
 		}
 		else
 		if(rb <= RULE_LAST_RULE)
@@ -1512,8 +1642,8 @@ void Translator::AppendPhonemes(char *string, int size, const char *ph)
 
 
 
-void Translator::MatchRule(char *word[], const char *group, char *rule, MatchRecord *match_out, int word_flags)
-{//==========================================================================================================
+void Translator::MatchRule(char *word[], const char *group, char *rule, MatchRecord *match_out, int word_flags, int dict_flags)
+{//============================================================================================================================
 /* Checks a specified word against dictionary rules.
 	Returns with phoneme code string, or NULL if no match found.
 
@@ -1550,6 +1680,8 @@ void Translator::MatchRule(char *word[], const char *group, char *rule, MatchRec
 	int  letter_group;
 	int  distance_right;
 	int  distance_left;
+	int  lg_pts;
+	int  n_bytes;
 
 	MatchRecord match;
 	static MatchRecord best;
@@ -1565,6 +1697,7 @@ void Translator::MatchRule(char *word[], const char *group, char *rule, MatchRec
 	if(rule == NULL)
 	{
 		match_out->points = 0;
+		(*word)++;
 		return;
 	}
 
@@ -1613,6 +1746,8 @@ void Translator::MatchRule(char *word[], const char *group, char *rule, MatchRec
 					if(common_phonemes != NULL)
 					{
 						match.phonemes = common_phonemes;
+						if(*match.phonemes == RULE_CONDITION)
+							match.phonemes += 2;   // skip over condition number
 						while(((rb = *match.phonemes++) != 0) && (rb != RULE_PHONEMES));
 					}
 					else
@@ -1638,9 +1773,21 @@ void Translator::MatchRule(char *word[], const char *group, char *rule, MatchRec
 				case RULE_CONDITION:
 					/* conditional rule, next byte gives condition number */
 					condition_num = *rule++;
-					if((dict_condition & (1L << condition_num))==0)
-						failed = 1;
+					
+					if(condition_num >= 32)
+					{
+						// allow the rule only if the condition number is NOT set
+						if((dict_condition & (1L << (condition_num-32))) != 0)
+							failed = 1;
+					}
 					else
+					{
+						// allow the rule only if the condition number is set
+						if((dict_condition & (1L << condition_num)) == 0)
+							failed = 1;
+					}
+
+					if(!failed)
 						match.points++;  // add one point for a matched conditional rule
 					break;
 				}
@@ -1679,13 +1826,27 @@ void Translator::MatchRule(char *word[], const char *group, char *rule, MatchRec
 					letter_group = *rule++ - 'A';
 					if(IsLetter(letter_w,letter_group))
 					{
-						match.points += (20-distance_right);
+						lg_pts = 20;
+						if(letter_group==2)
+							lg_pts = 19;  // fewer points for C, general consonant
+						match.points += (lg_pts-distance_right);
 						post_ptr += letter_xbytes;
 					}
 					else
 						failed = 1;
 					break;
 
+				case RULE_LETTERGP2:   // match against a list of utf-t strings
+					letter_group = *rule++ - 'A';
+					if((n_bytes = IsLetterGroup(post_ptr-1,letter_group)) >0)
+					{
+						match.points += (20-distance_right);
+						post_ptr += (n_bytes-1);
+					}
+					else
+						failed =1;
+					break;
+
 				case RULE_NOTVOWEL:
 					if(!IsLetter(letter_w,0))
 					{
@@ -1723,10 +1884,17 @@ void Translator::MatchRule(char *word[], const char *group, char *rule, MatchRec
 						failed = 1;
 					break;
 
+				case RULE_ALT1:
+					if(dict_flags & FLAG_ALT_TRANS)
+						match.points++;
+					else
+						failed = 1;
+					break;
+
 				case '-':
-					if((letter == ' ') && (word_flags & FLAG_HYPHEN_AFTER))
+					if((letter == '-') || ((letter == ' ') && (word_flags & FLAG_HYPHEN_AFTER)))
 					{
-						match.points += (21-distance_right);
+						match.points += (22-distance_right);    // one point more than match against space
 					}
 					else
 						failed = 1;
@@ -1761,6 +1929,23 @@ void Translator::MatchRule(char *word[], const char *group, char *rule, MatchRec
 					}
 					break;
 
+				case RULE_NOVOWELS:
+					{
+						char *p = post_ptr + letter_xbytes;
+						while(letter_w != RULE_SPACE)
+						{
+							if(IsLetter(letter_w,LETTERGP_VOWEL2))
+							{
+								failed = 1;
+								break;
+							}
+							p += utf8_in(&letter_w,p,0);
+						}
+						if(!failed)
+							match.points += (19-distance_right);
+					}
+					break;
+
 				case RULE_INC_SCORE:
 					match.points += 20;      // force an increase in points
 					break;
@@ -1786,6 +1971,8 @@ void Translator::MatchRule(char *word[], const char *group, char *rule, MatchRec
 				case RULE_NO_SUFFIX:
 					if(word_flags & FLAG_SUFFIX_REMOVED)
 						failed = 1;             // a suffix has been removed
+					else
+						match.points++;
 					break;
 
 				default:
@@ -1820,7 +2007,10 @@ void Translator::MatchRule(char *word[], const char *group, char *rule, MatchRec
 					letter_group = *rule++ - 'A';
 					if(IsLetter(letter_w,letter_group))
 					{
-						match.points += (20-distance_left);
+						lg_pts = 20;
+						if(letter_group==2)
+							lg_pts = 19;  // fewer points for C, general consonant
+						match.points += (lg_pts-distance_left);
 						pre_ptr -= letter_xbytes;
 					}
 					else
@@ -1885,6 +2075,23 @@ void Translator::MatchRule(char *word[], const char *group, char *rule, MatchRec
 						failed = 1;
 					break;
 
+				case RULE_NOVOWELS:
+					{
+						char *p = pre_ptr - letter_xbytes - 1;
+						while(letter_w != RULE_SPACE)
+						{
+							if(IsLetter(letter_w,LETTERGP_VOWEL2))
+							{
+								failed = 1;
+								break;
+							}
+							p -= utf8_in(&letter_w,p,1);
+						}
+						if(!failed)
+							match.points += 3;
+					}
+					break;
+
 				case RULE_IFVERB:
 					if(expect_verb)
 						match.points += 1;
@@ -1892,6 +2099,13 @@ void Translator::MatchRule(char *word[], const char *group, char *rule, MatchRec
 						failed = 1;
 					break;
 
+				case RULE_CAPITAL:
+					if(word_flags & FLAG_FIRST_UPPER)
+						match.points += 1;
+					else
+						failed = 1;
+					break;
+
 				case '.':
 					// dot in pre- section, match on any dot before this point in the word
 					for(p=pre_ptr; *p != ' '; p--)
@@ -1909,7 +2123,7 @@ void Translator::MatchRule(char *word[], const char *group, char *rule, MatchRec
 				case '-':
 					if((letter == '-') || ((letter == ' ') && (word_flags & FLAG_HYPHEN)))
 					{
-						match.points += (21-distance_right);
+						match.points += (22-distance_right);       // one point more than match against space
 					}
 					else
 						failed = 1;
@@ -1919,7 +2133,7 @@ void Translator::MatchRule(char *word[], const char *group, char *rule, MatchRec
 					if(letter == rb)
 					{
 						if(letter == RULE_SPACE)
-							match.points += 2;
+							match.points += 4;
 						else
 							match.points += (21-distance_left);
 					}
@@ -1941,7 +2155,7 @@ void Translator::MatchRule(char *word[], const char *group, char *rule, MatchRec
 			}
 
 #ifdef LOG_TRANSLATE
-			if((option_phonemes == 2) && (match.points > 0))
+			if((option_phonemes == 2) && (match.points > 0) && ((word_flags & FLAG_NO_TRACE) == 0))
 			{
 				// show each rule that matches, and it's points score
 				int pts;
@@ -1963,7 +2177,7 @@ void Translator::MatchRule(char *word[], const char *group, char *rule, MatchRec
 	}
 
 #ifdef LOG_TRANSLATE
-	if(option_phonemes == 2)
+	if((option_phonemes == 2) && ((word_flags & FLAG_NO_TRACE)==0))
 	{
 		if(group_length <= 1)
 			fprintf(f_trans,"\n");
@@ -1985,8 +2199,8 @@ void Translator::MatchRule(char *word[], const char *group, char *rule, MatchRec
 
 
 
-int Translator::TranslateRules(char *p, char *phonemes, int ph_size, char *end_phonemes, int word_flags, int dict_flags)
-{//=====================================================================================================================
+int Translator::TranslateRules(char *p_start, char *phonemes, int ph_size, char *end_phonemes, int word_flags, int dict_flags)
+{//===========================================================================================================================
 /* Translate a word bounded by space characters
    Append the result to 'phonemes' and any standard prefix/suffix in 'end_phonemes' */
 	
@@ -2001,11 +2215,13 @@ int Translator::TranslateRules(char *p, char *phonemes, int ph_size, char *end_p
 	int  g1;            /* first group for this letter */
 	int  n;
 	int  letter;
+	int  ix;
 	int  digit_count=0;
-	char *p_start;
+	char *p;
 	MatchRecord match1;
 	MatchRecord match2;
 	char ph_buf[40];
+	char word_copy[N_WORD_BYTES];
 	static const char str_pause[2] = {phonPAUSE_NOLINK,0};
 
 	char group_name[4];
@@ -2013,13 +2229,21 @@ int Translator::TranslateRules(char *p, char *phonemes, int ph_size, char *end_p
 	if(data_dictrules == NULL)
 		return(0);
 
+	for(ix=0; ix<(N_WORD_BYTES-1);)
+	{
+		c = p_start[ix];
+		word_copy[ix++] = c;
+	}
+	word_copy[ix] = 0;
+
+
 #ifdef LOG_TRANSLATE
-	if(option_phonemes == 2)
+	if((option_phonemes == 2) && ((word_flags & FLAG_NO_TRACE)==0))
 	{
 		char wordbuf[120];
 		int  ix;
 
-		for(ix=0; ((c = p[ix]) != ' ') && (c != 0); ix++)
+		for(ix=0; ((c = p_start[ix]) != ' ') && (c != 0); ix++)
 		{
 			wordbuf[ix] = c;
 		}
@@ -2028,7 +2252,7 @@ int Translator::TranslateRules(char *p, char *phonemes, int ph_size, char *end_p
 	}
 #endif
 
-	p_start = p;
+	p = p_start;
 	word_vowel_count = 0;
 	word_stressed_count = 0;
 	
@@ -2085,13 +2309,13 @@ int Translator::TranslateRules(char *p, char *phonemes, int ph_size, char *end_p
 						group_name[1] = c2;
 						group_name[2] = 0;
 						p2 = p;
-						MatchRule(&p2, group_name, groups2[g], &match2, word_flags);
+						MatchRule(&p2, group_name, groups2[g], &match2, word_flags, dict_flags);
 						if(match2.points > 0)
 							match2.points += 35;   /* to acount for 2 letters matching */
 
 						/* now see whether single letter chain gives a better match ? */
 						group_name[1] = 0;
-						MatchRule(&p, group_name, groups1[c], &match1, word_flags);
+						MatchRule(&p, group_name, groups1[c], &match1, word_flags, dict_flags);
 
 						if(match2.points >= match1.points)
 						{
@@ -2110,11 +2334,11 @@ int Translator::TranslateRules(char *p, char *phonemes, int ph_size, char *end_p
 				group_name[1] = 0;
 	
 				if(groups1[c] != NULL)
-					MatchRule(&p, group_name, groups1[c], &match1, word_flags);
+					MatchRule(&p, group_name, groups1[c], &match1, word_flags, dict_flags);
 				else
 				{
 					// no group for this letter, use default group
-					MatchRule(&p, "", groups1[0],&match1,word_flags);
+					MatchRule(&p, "", groups1[0], &match1, word_flags, dict_flags);
 
 					if(match1.points == 0)
 					{
@@ -2171,8 +2395,21 @@ int Translator::TranslateRules(char *p, char *phonemes, int ph_size, char *end_p
 			if((match1.end_type != 0) && (end_phonemes != NULL))
 			{
 				/* a standard ending has been found, re-translate the word without it */
-				strcpy(end_phonemes,match1.phonemes);
-				return(match1.end_type);
+				if((match1.end_type & SUFX_P) && (word_flags & FLAG_NO_PREFIX))
+				{
+					// ignore the match on a prefix
+				}
+				else
+				{
+					if((match1.end_type & SUFX_P) && ((match1.end_type & 0x7f) == 0))
+					{
+						// no prefix length specified
+						match1.end_type |= p - p_start;
+					}
+					strcpy(end_phonemes,match1.phonemes);
+					memcpy(p_start,word_copy,strlen(word_copy));
+					return(match1.end_type);
+				}
 			}
 			if(match1.del_fwd != NULL)
 				*match1.del_fwd = REPLACED_E;
@@ -2182,6 +2419,7 @@ int Translator::TranslateRules(char *p, char *phonemes, int ph_size, char *end_p
 
 	// any language specific changes ?
 	ApplySpecialAttribute(phonemes,dict_flags);
+	memcpy(p_start,word_copy,strlen(word_copy));
 	return(0);
 }   /* end of TranslateRules */
 
@@ -2346,8 +2584,9 @@ int TransposeAlphabet(char *text, int offset, int min, int max)
 			*p2++ = (acc << (8-bits));
 		}
 		*p2 = 0;
+		return((p2 - text) | 0x40);  // bit 6 indicates compressed characters
 	}
-	return((p2 - text) | 0x40);  // bit 6 indicates compressed characters
+	return(p2 - text);
 }  // end of TransposeAlphabet
 
 
@@ -2411,6 +2650,9 @@ int Translator::LookupDict2(char *word, char *word2, char *phonetic, unsigned in
 		return(0);
 	}
 
+	// Find the first entry in the list for this hash value which matches.
+	// This corresponds to the last matching entry in the *_list file.
+
 	while(*p != 0)
 	{
 		next = p + p[0];
@@ -2450,8 +2692,18 @@ int Translator::LookupDict2(char *word, char *word2, char *phonetic, unsigned in
 			if(flag >= 100)
 			{
 				// conditional rule
-				if((dict_condition & (1 << (flag-100))) == 0)
-					condition_failed = 1;
+				if(flag >= 132)
+				{
+					// fail if this condition is set
+					if((dict_condition & (1 << (flag-132))) != 0)
+						condition_failed = 1;
+				}
+				else
+				{
+					// allow only if this condition is set
+					if((dict_condition & (1 << (flag-100))) == 0)
+						condition_failed = 1;
+				}
 			}
 			else
 			if(flag > 64)
@@ -2598,23 +2850,28 @@ int Translator::LookupDictList(char *word1, char *ph_out, unsigned int *flags, i
 	int  found;
 	char *word2;
 	unsigned char c;
+	int  nbytes;
+	int  c2;
 	char word[N_WORD_BYTES];
 
 	length = 0;
 	word2 = word1;
-	while((word2[1]==' ') && (word2[2]=='.'))
+
+	while((word2[nbytes = utf8_in(&c2,word2,0)]==' ') && (word2[nbytes+1]=='.'))
 	{
 		// look for an abbreviation of the form a.b.c
 		// try removing the spaces between the dots and looking for a match
-		word[length++] = word2[0];
+		memcpy(&word[length],word2,nbytes);
+		length += nbytes;
 		word[length++] = '.';
-		word2 += 4;
+		word2 += nbytes+3;
 	}
 	if(length > 0)
 	{
 		// found an abbreviation containing dots
-		word[length] = word2[0];
-		word[length+1] = 0;
+		nbytes = utf8_in(&c2,word2,0);
+		memcpy(&word[length],word2,nbytes);
+		word[length+nbytes] = 0;
 		found =  LookupDict2(word,word2,ph_out,flags,end_flags);
 		if(found)
 		{
@@ -2662,7 +2919,8 @@ int Translator::LookupDictList(char *word1, char *ph_out, unsigned int *flags, i
 
 int Translator::Lookup(char *word, char *ph_out)
 {//=============================================
-	return(LookupDictList(word,ph_out,NULL,0));
+	unsigned int flags;
+	return(LookupDictList(word,ph_out,&flags,0));
 }
 
 
