@@ -1,10 +1,10 @@
 /***************************************************************************
- *   Copyright (C) 2005,2006 by Jonathan Duddington                        *
- *   jsd@clara.co.uk                                                       *
+ *   Copyright (C) 2005 to 2007 by Jonathan Duddington                     *
+ *   email: jonsd@users.sourceforge.net                                    *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
- *   the Free Software Foundation; either version 2 of the License, or     *
+ *   the Free Software Foundation; either version 3 of the License, or     *
  *   (at your option) any later version.                                   *
  *                                                                         *
  *   This program is distributed in the hope that it will be useful,       *
@@ -13,12 +13,12 @@
  *   GNU General Public License for more details.                          *
  *                                                                         *
  *   You should have received a copy of the GNU General Public License     *
- *   along with this program; if not, write to the                         *
- *   Free Software Foundation, Inc.,                                       *
- *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
+ *   along with this program; if not, see:                                 *
+ *               <http://www.gnu.org/licenses/>.                           *
  ***************************************************************************/
 
 
+
 // phoneme types
 #define phPAUSE   0
 #define phSTRESS  1
@@ -30,6 +30,7 @@
 #define phVFRICATIVE 7
 #define phNASAL   8
 #define phVIRTUAL 9
+#define phDELETED 14
 #define phINVALID 15
 
 
@@ -45,6 +46,8 @@
 #define phVOWEL2   0x100   // liquid that is considered a vowel
 #define phPALATAL  0x200
 #define phLONG     0x1000
+#define phAPPENDPH 0x2000  // always insert another phoneme (link_out) after this one
+#define phBRKAFTER 0x4000  // [*] add a post-pause
 
 #define phALTERNATIVE    0x0c00   // bits 10,11  specifying use of alternative_ph
 #define phBEFOREVOWEL    0x0000
@@ -52,6 +55,8 @@
 #define phBEFORENOTVOWEL 0x0c00
 #define phSWITCHVOICING  0x0800
 
+#define phNONSYLLABIC 0x100000   // don't count this vowel as a syllable when finding the stress position 
+
 // fixed phoneme code numbers, these can be used from the program code
 #define phonCONTROL     1
 #define phonSTRESS_U    2
@@ -75,14 +80,16 @@
 #define phonSYLLABIC    20
 #define phonSWITCH      21
 #define phonX1          22      // a language specific action
+#define phonPAUSE_VSHORT 23
+#define phonT_REDUCED   24
 
 
 // place of articulation
 #define phPLACE        0xf0000
 #define phPLACE_pla    0x60000
 
-#define N_PHONEME_TABS      50
-#define N_PHONEME_TAB      256
+#define N_PHONEME_TABS     100     // number of phoneme tables
+#define N_PHONEME_TAB      256     // max phonemes in a phoneme table
 #define N_PHONEME_TAB_NAME  32     // must be multiple of 4
 
 // main table of phonemes, index by phoneme number (1-254)
@@ -113,12 +120,13 @@ typedef struct {
 // one for the current voice
 extern int n_phoneme_tab;
 extern PHONEME_TAB *phoneme_tab[N_PHONEME_TAB];
+extern unsigned char phoneme_tab_flags[N_PHONEME_TAB];  // bit 0: not inherited
 
 typedef struct {
 	char name[N_PHONEME_TAB_NAME];
 	PHONEME_TAB *phoneme_tab_ptr;
 	int n_phonemes;
-	int includes;
+	int includes;           // also include the phonemes from this other phoneme table
 } PHONEME_TAB_LIST;
 
 
@@ -136,7 +144,9 @@ extern REPLACE_PHONEMES replace_phonemes[N_REPLACE_PHONEMES];
 
 
 
-char *EncodePhonemes(char *p, char *outptr, char *bad_phoneme);
+char *EncodePhonemes(char *p, char *outptr, unsigned char *bad_phoneme);
 void DecodePhonemes(const char *inptr, char *outptr);
 const char *PhonemeTabName(void);
+int LookupPh(const char *string);
+extern const char *WordToString(unsigned int word);
 
