@@ -1,10 +1,10 @@
 /***************************************************************************
- *   Copyright (C) 2005,2006 by Jonathan Duddington                        *
- *   jonsd@users.sourceforge.net                                           *
+ *   Copyright (C) 2005 to 2007 by Jonathan Duddington                     *
+ *   email: jonsd@users.sourceforge.net                                    *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
- *   the Free Software Foundation; either version 2 of the License, or     *
+ *   the Free Software Foundation; either version 3 of the License, or     *
  *   (at your option) any later version.                                   *
  *                                                                         *
  *   This program is distributed in the hope that it will be useful,       *
@@ -13,10 +13,10 @@
  *   GNU General Public License for more details.                          *
  *                                                                         *
  *   You should have received a copy of the GNU General Public License     *
- *   along with this program; if not, write to the                         *
- *   Free Software Foundation, Inc.,                                       *
- *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
+ *   along with this program; if not, see:                                 *
+ *               <http://www.gnu.org/licenses/>.                           *
  ***************************************************************************/
+
 #include "StdAfx.h"
 
 #include <stdio.h>
@@ -28,9 +28,9 @@
 
 #include "speak_lib.h"
 #include "speech.h"
-#include "voice.h"
 #include "phoneme.h"
 #include "synthesize.h"
+#include "voice.h"
 #include "translate.h"
 
 #define    PITCHfall   0
@@ -45,6 +45,7 @@ static void SmoothSpect(void);
 int n_phoneme_list=0;
 PHONEME_LIST phoneme_list[N_PHONEME_LIST];
 
+char mbrola_name[20];
 
 int speed_factor1;
 int speed_factor2;
@@ -66,10 +67,8 @@ static voice_t *new_voice=NULL;
 int n_soundicon_tab=0;
 SOUND_ICON soundicon_tab[N_SOUNDICON_TAB];
 
-#define RMS1  16      //
-#define RMS2  20      // 16 - 20
 #define RMS_GLOTTAL1 35   // vowel before glottal stop
-#define RMS_START  25  // 14 - 30
+#define RMS_START  28  // 14 - 30
 
 #define VOWEL_FRONT_LENGTH  50
 
@@ -188,13 +187,20 @@ static void DoPitch(unsigned char *env, int pitch1, int pitch2)
 
 
 
+int PauseLength(int pause)
+{//=======================
+	int len;
+	len = (pause * speed_factor1)/256;
+	if(len < 5) len = 5;      // mS, limit the amount to which pauses can be shortened
+	return(len);
+}
+
+
 static void DoPause(int length)
 {//============================
 	int len;
 
-	len = length * speed_factor1;
-	if(len < 1000) len = 1000;      // limit the amount to which pauses can be shortened
-	len = len / 256;
+	len = PauseLength(length);
 
 	len = (len * samplerate) / 1000;  // convert from mS to number of samples
 
@@ -209,8 +215,8 @@ static void DoPause(int length)
 extern int seq_len_adjust;   // temporary fix to advance the start point for playing the wav sample
 
 
-static void DoSample2(int index, int which, int length_mod, int amp)
-{//=================================================================
+static int DoSample2(int index, int which, int length_mod, int amp)
+{//================================================================
 	int length;
 	int length1;
 	int format;
@@ -249,37 +255,45 @@ static void DoSample2(int index, int which, int length_mod, int amp)
 
 	index += 4;
 
-	last_wcmdq = wcmdq_tail;
-	q = wcmdq[wcmdq_tail];
-	if(which & 0x100)
-		q[0] = WCMD_WAVE2;    // mix this with synthesised wave
-	else
-		q[0] = WCMD_WAVE;
-	q[1] = length;   // length in samples
-	q[2] = long(&wavefile_data[index]);
-	q[3] = format + (amp << 8);
-	WcmdqInc();
-
+	if(amp >= 0)
+	{
+		last_wcmdq = wcmdq_tail;
+		q = wcmdq[wcmdq_tail];
+		if(which & 0x100)
+			q[0] = WCMD_WAVE2;    // mix this with synthesised wave
+		else
+			q[0] = WCMD_WAVE;
+		q[1] = length;   // length in samples
+		q[2] = long(&wavefile_data[index]);
+		q[3] = format + (amp << 8);
+		WcmdqInc();
+	}
+	return(length);
 }  // end of Synthesize::DoSample2
 
 
-static void DoSample(PHONEME_TAB *ph1, PHONEME_TAB *ph2, int which, int length_mod, int amp)
-{//=========================================================================================
+int DoSample(PHONEME_TAB *ph1, PHONEME_TAB *ph2, int which, int length_mod, int amp)
+{//====================== ==========================================================
 	int index;
 	int match_level;
 	int amp2;
+	int result;
 
 	EndPitch(1);
 	index = LookupSound(ph1,ph2,which & 0xff,&match_level,0);
 	if((index & 0x800000) == 0)
-		return;             // not wavefile data
+		return(0);             // not wavefile data
 
 	amp2 = wavefile_amp;
 	if(amp != 0)
 		amp2 = (amp * wavefile_amp)/20;
 
-	DoSample2(index,which,length_mod,amp2);
+	if(amp == -1)
+		amp2 = amp;
+
+	result = DoSample2(index,which,length_mod,amp2);
 	last_frame = NULL;
+	return(result);
 }  // end of Synthesize::DoSample
 
 
@@ -311,18 +325,24 @@ static void set_frame_rms(frame_t *fr, int new_rms)
 	int h;
 	int ix;
 
-	static const short sqrt_tab[100] = {
-     0, 64, 90,110,128,143,156,169,181,192,202,212,221,230,239,247,
-   256,263,271,278,286,293,300,306,313,320,326,332,338,344,350,356,
-   362,367,373,378,384,389,394,399,404,409,414,419,424,429,434,438,
-   443,448,452,457,461,465,470,474,478,483,487,491,495,499,503,507,
-   512,515,519,523,527,531,535,539,543,546,550,554,557,561,565,568,
-   572,576,579,583,586,590,593,596,600,603,607,610,613,617,620,623,
-   627,630,633,636 };
+	static const short sqrt_tab[200] = {
+	  0, 64, 90,110,128,143,156,169,181,192,202,212,221,230,239,247,
+	256,263,271,278,286,293,300,306,313,320,326,332,338,344,350,356,
+	362,367,373,378,384,389,394,399,404,409,414,419,424,429,434,438,
+	443,448,452,457,461,465,470,474,478,483,487,491,495,499,503,507,
+	512,515,519,523,527,531,535,539,543,546,550,554,557,561,565,568,
+	572,576,579,583,586,590,593,596,600,603,607,610,613,617,620,623,
+	627,630,633,636,640,643,646,649,652,655,658,662,665,668,671,674,
+	677,680,683,686,689,692,695,698,701,704,706,709,712,715,718,721,
+	724,726,729,732,735,738,740,743,746,749,751,754,757,759,762,765,
+	768,770,773,775,778,781,783,786,789,791,794,796,799,801,804,807,
+	809,812,814,817,819,822,824,827,829,832,834,836,839,841,844,846,
+	849,851,853,856,858,861,863,865,868,870,872,875,877,879,882,884,
+	886,889,891,893,896,898,900,902};
 
 	if(fr->rms == 0) return;    // check for divide by zero
 	x = (new_rms * 64)/fr->rms;
-	if(x >= 100) x = 99;
+	if(x >= 200) x = 199;
 
 	x = sqrt_tab[x];   // sqrt(new_rms/fr->rms)*0x200;
 
@@ -389,6 +409,8 @@ static void AdjustFormants(frame_t *fr, int target, int min, int max, int f1_adj
 
 //hf_reduce = 70;      // ?? using fixed amount rather than the parameter??
 
+	target = (target * voice->formant_factor)/256;
+
 	x = (target - fr->ffreq[2]) / 2;
 	if(x > max) x = max;
 	if(x < min) x = min;
@@ -448,6 +470,7 @@ int FormantTransition2(frameref_t *seq, int &n_frames, unsigned int data1, unsig
 {//==============================================================================================================================
 	int ix;
 	int formant;
+	int next_rms;
 
 	int len;
 	int rms;
@@ -473,7 +496,7 @@ static short vcolouring[N_VCOLOUR][5] = {
 		return(0);
 
 	len = (data1 & 0x3f) * 2;
-	rms = ((data1 >> 6) & 0x3f) * 2;
+	rms = (data1 >> 6) & 0x3f;
 	flags = (data1 >> 12);
 
 	f2 = (data2 & 0x3f) * 50;
@@ -501,26 +524,39 @@ static short vcolouring[N_VCOLOUR][5] = {
 		seq[0].frflags |= FRFLAG_LEN_MOD;              // reduce length modification
 		fr->frflags |= FRFLAG_LEN_MOD;
 
+		next_rms = seq[1].frame->rms;
+
 		if(f2 != 0)
 		{
+			if(rms & 0x20)
+			{
+				set_frame_rms(fr,(next_rms * (rms & 0x1f))/30);
+			}
 			AdjustFormants(fr, f2, f2_min, f2_max, f1, f3_adj, f3_amp, flags);
-			set_frame_rms(fr,rms);
+
+			if((rms & 0x20) == 0)
+			{
+				set_frame_rms(fr,rms*2);
+			}
 		}
 		else
 		{
-			set_frame_rms(fr,RMS_START);
+			if(flags & 8)
+				set_frame_rms(fr,(next_rms*24)/32);
+			else
+				set_frame_rms(fr,RMS_START);
 		}
 
 		if(flags & 8)
 		{
-			set_frame_rms(fr,seq[1].frame->rms - 5);
+//			set_frame_rms(fr,next_rms - 5);
 			modn_flags = 0x800 + (VowelCloseness(fr) << 8);
 		}
 	}
 	else
 	{
 		// exit from vowel
-
+		rms = rms*2;
 		if((f2 != 0) || (flags != 0))
 		{
 
@@ -636,14 +672,37 @@ static void SmoothSpect(void)
 			frame = frame2 = (frame_t *)q[2];
 			modified = 0;
 
+			if(frame->frflags & FRFLAG_BREAK)
+				break;
+
 			if(frame->frflags & FRFLAG_FORMANT_RATE)
-				len = (len * 13)/10;      // allow slightly greater rate of change for this frame (was 12/10)
+				len = (len * 12)/10;      // allow slightly greater rate of change for this frame (was 12/10)
 
 			for(pk=0; pk<6; pk++)
 			{
-				allowed = (formant_rate[pk] * len)/256;
+				int f1, f2;
+
+				if((frame->frflags & FRFLAG_BREAK_LF) && (pk < 3))
+					continue;
+
+				f1 = frame1->ffreq[pk];
+				f2 = frame->ffreq[pk];
+
+				// backwards
+				if((diff = f2 - f1) > 0)
+				{
+					allowed = f1*2 + f2;
+				}
+				else
+				{
+					allowed = f1 + f2*2;
+				}
+
+				// the allowed change is specified as percentage (%*10) of the frequency
+				// take "frequency" as 1/3 from the lower freq
+				allowed = (allowed * formant_rate[pk])/3000;
+				allowed = (allowed * len)/256;
 
-				diff = frame->ffreq[pk] - frame1->ffreq[pk];
 				if(diff > allowed)
 				{
 					if(modified == 0)
@@ -704,14 +763,30 @@ static void SmoothSpect(void)
 			frame = frame2 = (frame_t *)q[3];
 			modified = 0;
 
-			if(frame->frflags & FRFLAG_FORMANT_RATE)
+			if(frame1->frflags & FRFLAG_BREAK)
+				break;
+
+			if(frame1->frflags & FRFLAG_FORMANT_RATE)
 				len = (len *6)/5;      // allow slightly greater rate of change for this frame
 
 			for(pk=0; pk<6; pk++)
 			{
-				allowed = (formant_rate[pk] * len)/256;
+				int f1, f2;
+				f1 = frame1->ffreq[pk];
+				f2 = frame->ffreq[pk];
+
+				// forwards
+				if((diff = f2 - f1) > 0)
+				{
+					allowed = f1*2 + f2;
+				}
+				else
+				{
+					allowed = f1 + f2*2;
+				}
+				allowed = (allowed * formant_rate[pk])/3000;
+				allowed = (allowed * len)/256;
 
-				diff = frame->ffreq[pk] - frame1->ffreq[pk];
 				if(diff > allowed)
 				{
 					if(modified == 0)
@@ -754,11 +829,12 @@ static void StartSyllable(void)
 }
 
 
-static void DoSpect(PHONEME_TAB *this_ph, PHONEME_TAB *prev_ph, PHONEME_TAB *next_ph,
+int DoSpect(PHONEME_TAB *this_ph, PHONEME_TAB *prev_ph, PHONEME_TAB *next_ph,
 		int which, PHONEME_LIST *plist, int modulation)
 {//===================================================================================
 	// which  1  start of phoneme,   2 body and end
 	// length_mod: 256 = 100%
+	// modulation: -1 = don't write to wcmdq
 
 	int  n_frames;
 	frameref_t *frames;
@@ -775,6 +851,7 @@ static void DoSpect(PHONEME_TAB *this_ph, PHONEME_TAB *prev_ph, PHONEME_TAB *nex
 	int  frame2_length;
 	int  length_factor;
 	int  length_mod;
+	int  total_len = 0;
 	static int wave_flag = 0;
 	int wcmd_spect = WCMD_SPECT;
 
@@ -796,7 +873,7 @@ if(which==1)
 	modn_flags = 0;
 	frames = LookupSpect(this_ph,prev_ph,next_ph,which,&match_level,&n_frames, plist);
 	if(frames == NULL)
-		return;   // not found
+		return(0);   // not found
 
 	if(wavefile_ix == 0)
 	{
@@ -862,17 +939,20 @@ if(which==1)
 		length_factor = length_mod;
 		if(frame1->frflags & FRFLAG_LEN_MOD)     // reduce effect of length mod
 		{
-			length_factor = (length_mod + 256)/2;
-		}
-		if(frame1->frflags & FRFLAG_MODULATE)
-		{
-			modulation = 6;
+			length_factor = (length_mod*4 + 256*3)/7;
 		}
 		len = (frame_length * samplerate)/1000;
 		len = (len * length_factor)/256;
 
-		if((frameix == n_frames-1) && (modn_flags & 0xf00))
-			modulation |= modn_flags;   // before or after a glottal stop
+		if(modulation >= 0)
+		{
+			if(frame1->frflags & FRFLAG_MODULATE)
+			{
+				modulation = 6;
+			}
+			if((frameix == n_frames-1) && (modn_flags & 0xf00))
+				modulation |= modn_flags;   // before or after a glottal stop
+		}
 
 		pitch_length += len;
 		amp_length += len;
@@ -886,17 +966,23 @@ if(which==1)
 		else
 		{
 			last_wcmdq = wcmdq_tail;
-			q = wcmdq[wcmdq_tail];
-			q[0] = wcmd_spect;
-			q[1] = len + (modulation << 16);
-			q[2] = long(frame1);
-			q[3] = long(frame2);
 
-			WcmdqInc();
+			if(modulation >= 0)
+			{
+				q = wcmdq[wcmdq_tail];
+				q[0] = wcmd_spect;
+				q[1] = len + (modulation << 16);
+				q[2] = long(frame1);
+				q[3] = long(frame2);
+	
+				WcmdqInc();
+			}
 			last_frame = frame1 = frame2;
 			frame_length = frame2_length;
+			total_len += len;
 		}
 	}
+	return(total_len);
 }  // end of Synthesize::DoSpect
 
 
@@ -915,8 +1001,13 @@ static void DoMarker(int type, int char_posn, int length, int value)
 
 static void DoVoice(voice_t *v)
 {//============================
+// allocate memory for a copy of the voice data, and free it in wavegenfill()
+	voice_t *v2;
+
+	v2 = (voice_t *)malloc(sizeof(voice_t));
+	memcpy(v2,v,sizeof(voice_t));
 	wcmdq[wcmdq_tail][0] = WCMD_VOICE;
-	wcmdq[wcmdq_tail][1] = (long)v;
+	wcmdq[wcmdq_tail][1] = (long)(v2);
 	WcmdqInc();
 }
 
@@ -984,6 +1075,7 @@ int Generate(PHONEME_LIST *phoneme_list, int *n_ph, int resume)
 {//============================================================
 	static int  ix;
 	static int  embedded_ix;
+	static int  word_count;
 	PHONEME_LIST *prev;
 	PHONEME_LIST *next;
 	PHONEME_LIST *next2;
@@ -992,15 +1084,24 @@ int Generate(PHONEME_LIST *phoneme_list, int *n_ph, int resume)
 	int  stress;
 	int  modulation;
 	int  pre_voiced;
-	int  word_count = 0;
 	unsigned char *pitch_env=NULL;
 	unsigned char *amp_env;
 	PHONEME_TAB *ph;
+	PHONEME_TAB *prev_ph;
+
+#ifdef TEST_MBROLA
+	if(mbrola_name[0] != 0)
+		return(MbrolaGenerate(phoneme_list,n_ph,resume));
+#endif
+
+	if(option_quiet)
+		return(0);
 
 	if(resume == 0)
 	{
 		ix = 1;
 		embedded_ix=0;
+		word_count = 0;
 		pitch_length = 0;
 		amp_length = 0;
 		last_frame = NULL;
@@ -1056,6 +1157,11 @@ int Generate(PHONEME_LIST *phoneme_list, int *n_ph, int resume)
 		if(p->prepause > 0)
 			DoPause(p->prepause);
 
+		if(option_phoneme_events)
+		{
+			DoMarker(espeakEVENT_PHONEME, (p->sourceix & 0x7ff) + clause_start_char, 0, p->ph->mnemonic);
+		}
+
 		switch(p->type)
 		{
 		case phPAUSE:
@@ -1113,6 +1219,13 @@ int Generate(PHONEME_LIST *phoneme_list, int *n_ph, int resume)
 					DoSpect(p->ph,phoneme_tab[phonSCHWA],next->ph,1,p,0);
 				}
 			}
+			else
+			{
+				if(p->synthflags & SFLAG_LENGTHEN)
+				{
+					DoPause(50);
+				}
+			}
 
 			if(pre_voiced)
 			{
@@ -1151,7 +1264,7 @@ int Generate(PHONEME_LIST *phoneme_list, int *n_ph, int resume)
 				}
 			}
 
-			if((next->type==phVOWEL) || (next->type==phLIQUID))
+			if((next->type==phVOWEL) || ((next->type==phLIQUID)) && (next->newword==0))  // ?? test 14.Aug.2007
 			{
 				StartSyllable();
 				if(p->synthflags & SFLAG_LENGTHEN)
@@ -1205,6 +1318,10 @@ int Generate(PHONEME_LIST *phoneme_list, int *n_ph, int resume)
 			if(p->ph->phflags & phTRILL)
 				modulation = 5;
 
+			prev_ph = prev->ph;
+//			if(p->newword)
+//				prev_ph = phoneme_tab[phonPAUSE];    // pronounce fully at the start of a word
+
 			if(!(p->synthflags & SFLAG_SEQCONTINUE))
 			{
 				DoAmplitude(p->amp,NULL);
@@ -1219,16 +1336,16 @@ int Generate(PHONEME_LIST *phoneme_list, int *n_ph, int resume)
 			if(next->type==phVOWEL)
 			{
 				StartSyllable();
-				DoSpect(p->ph,prev->ph,next->ph,1,p,modulation);  // (,)r
+				DoSpect(p->ph,prev_ph,next->ph,1,p,modulation);  // (,)r
 			}
 			else
 			if(prev->type==phVOWEL && (p->synthflags & SFLAG_SEQCONTINUE))
 			{
-				DoSpect(p->ph,prev->ph,next->ph,1,p,modulation);
+				DoSpect(p->ph,prev_ph,next->ph,1,p,modulation);
 			}
 			else
 			{
-				DoSpect(p->ph,prev->ph,next->ph,1,p,modulation);
+				DoSpect(p->ph,prev_ph,next->ph,1,p,modulation);
 			}
 
 			break;
@@ -1435,6 +1552,11 @@ int SpeakNextClause(FILE *f_in, const void *text_in, int control)
 	{
 		fprintf(f_trans,"%s\n",translator->phon_out);
 	}
+	if(phoneme_callback != NULL)
+	{
+		phoneme_callback(translator->phon_out);
+	}
+
 	translator->CalcPitches(clause_tone);
 	translator->CalcLengths();
 
@@ -1452,6 +1574,15 @@ int SpeakNextClause(FILE *f_in, const void *text_in, int control)
 		return(1);
 	}
 
+	if(mbrola_name[0] != 0)
+	{
+#ifdef USE_MBROLA_LIB
+		MbrolaTranslate(phoneme_list,n_phoneme_list,NULL);
+#else
+		MbrolaTranslate(phoneme_list,n_phoneme_list,stdout);
+#endif
+	}
+
 	Generate(phoneme_list,&n_phoneme_list,0);
 	WavegenOpenSound();
 
