@@ -1,10 +1,10 @@
 /***************************************************************************
- *   Copyright (C) 2005,2006 by Jonathan Duddington                        *
- *   jonsd@users.sourceforge.net                                           *
+ *   Copyright (C) 2005 to 2007 by Jonathan Duddington                     *
+ *   email: jonsd@users.sourceforge.net                                    *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
- *   the Free Software Foundation; either version 2 of the License, or     *
+ *   the Free Software Foundation; either version 3 of the License, or     *
  *   (at your option) any later version.                                   *
  *                                                                         *
  *   This program is distributed in the hope that it will be useful,       *
@@ -13,11 +13,11 @@
  *   GNU General Public License for more details.                          *
  *                                                                         *
  *   You should have received a copy of the GNU General Public License     *
- *   along with this program; if not, write to the                         *
- *   Free Software Foundation, Inc.,                                       *
- *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
+ *   along with this program; if not, see:                                 *
+ *               <http://www.gnu.org/licenses/>.                           *
  ***************************************************************************/
 
+
 #define L(c1,c2)  (c1<<8)+c2          // combine two characters into an integer for translator name 
 
 #define CTRL_EMBEDDED    0x01         // control character at the start of an embedded command
@@ -28,11 +28,14 @@
 #define N_CLAUSE_WORDS   256          // max words in a clause
 #define N_RULE_GROUP2    120          // max num of two-letter rule chains
 #define N_HASH_DICT     1024
-#define N_CHARSETS        19
+#define N_CHARSETS        20
+#define N_LETTER_GROUPS   20
+
 
 /* flags from word dictionary */
 // bits 0-3  stressed syllable,  7=unstressed
 #define FLAG_SKIPWORDS        0xe0  /* bits 5,6,7  number of words to skip */
+#define FLAG_SKIPWORDS_1      0x20
 #define FLAG_PREPAUSE        0x100
 #define FLAG_ONLY            0x200
 #define BITNUM_FLAG_ONLY         9  // bit 9 is set
@@ -45,7 +48,7 @@
 
 #define FLAG_CAPITAL        0x8000  /* pronunciation if initial letter is upper case */
 #define FLAG_DOT           0x10000  /* ignore '.' after word (abbreviation) */
-#define FLAG_ABBREV        0x20000  /* use this pronunciation rather than split into letters */
+#define FLAG_ABBREV        0x20000  // spell as letters, even with a vowel, OR use specified pronunciation rather than split into letters
 #define FLAG_STEM          0x40000  // must have a suffix
 
 #define FLAG_DOUBLING      0x80000  // doubles the following consonant
@@ -66,15 +69,19 @@
 // wordflags, flags in source word
 #define FLAG_ALL_UPPER     0x1    /* no lower case letters in the word */
 #define FLAG_FIRST_UPPER   0x2    /* first letter is upper case */
+#define FLAG_UPPERS        0x3    // FLAG_ALL_UPPER | FLAG_FIRST_UPPER
 #define FLAG_HAS_PLURAL    0x4    /* upper-case word with s or 's lower-case ending */
 #define FLAG_PHONEMES      0x8    /* word is phonemes */
 #define FLAG_LAST_WORD     0x10   /* last word in clause */
 #define FLAG_STRESSED_WORD 0x20   /* this word has explicit stress */
 #define FLAG_EMBEDDED      0x40   /* word is preceded by embedded commands */
 #define FLAG_HYPHEN        0x80
+#define FLAG_NOSPACE       0x100  // word is not seperated from previous word by a space
 #define FLAG_DONT_SWITCH_TRANSLATOR  0x1000
 #define FLAG_SUFFIX_REMOVED  0x2000
 #define FLAG_HYPHEN_AFTER    0x4000
+#define FLAG_NO_PREFIX      0x8000
+#define FLAG_NO_TRACE      0x10000
 
 // prefix/suffix flags (bits 8 to 14, bits 16 to 22) don't use 0x8000, 0x800000
 #define SUFX_E        0x0100   // e may have been added
@@ -109,19 +116,16 @@
 #define RULE_ENDING		14
 #define RULE_DIGIT		15   // D digit
 #define RULE_NONALPHA	16   // Z non-alpha
-#define RULE_LETTER_GROUPS 17  // 17 to 23
-#define RULE_LETTER1		17   // A vowels
-#define RULE_LETTER2		18   // B 'hard' consonants 
-#define RULE_LETTER3		19   // C all consonants
-#define RULE_LETTER4		20   // H letter group
-#define RULE_LETTER5    21   // F letter group
-#define RULE_LETTER6		22   // G letter group
-#define RULE_LETTER7    23   // Y letter group
+#define RULE_LETTERGP   17   // A B C H F G Y   letter group number
+#define RULE_LETTERGP2  18   // L + letter group number
+#define RULE_CAPITAL    19   //    word starts with a capital letter
 #define RULE_NO_SUFFIX  24   // N
 #define RULE_NOTVOWEL   25   // K
 #define RULE_IFVERB     26   // V
-#define RULE_LETTERGP   27   // L + letter group number
-#define RULE_LAST_RULE   27
+#define RULE_ALT1       28   // T word has $alt attribute
+#define RULE_NOVOWELS   29   // X no vowels up to word boundary
+#define RULE_SPELLING   31   // W while spelling letter-by-letter
+#define RULE_LAST_RULE   31
 
 #define LETTERGP_A	0
 #define LETTERGP_B	1
@@ -134,7 +138,7 @@
 
 
 // Punctuation types  returned by ReadClause()
-// bits 0-7 pause x 10mS, bits 8-11 intonation type,
+// bits 0-7 pause x 10mS, bits 8-10 intonation type,
 // bit 15=sentence, bit 14=clause,  bits 13=voice change
 // bit 12 used to distinguish otherwise identical types
 #define CLAUSE_BIT_SENTENCE  0x8000
@@ -146,6 +150,7 @@
 #define CLAUSE_VOICE        0 + 0x2400
 #define CLAUSE_PERIOD      35 + 0x8000
 #define CLAUSE_COMMA       20 + 0x4100
+#define CLAUSE_SHORTCOMMA  15 + 0x4100
 #define CLAUSE_QUESTION    35 + 0x8200
 #define CLAUSE_EXCLAMATION 40 + 0x8300
 #define CLAUSE_COLON       30 + 0x4000
@@ -206,9 +211,10 @@ extern const int param_defaults[N_SPEECH_PARAM];
 
 
 
-#define N_LOPTS      12
+#define N_LOPTS      13
 #define LOPT_DIERESES        1
  // 1=remove [:] from unstressed syllables, 2= remove from unstressed or non-penultimate syllables
+ // bit 4=0, if stress < 4,  bit 4=1, if not the highest stress in the word
 #define LOPT_IT_LENGTHEN        2
  // 1=german
 #define LOPT_PREFIXES        3
@@ -223,12 +229,21 @@ extern const int param_defaults[N_SPEECH_PARAM];
 #define LOPT_LENGTH_MODS    6
  // increase this to prevent sonorants being shortened before shortened (eg. unstressed) vowels
 #define LOPT_SONORANT_MIN    7
- // Italian "syntactic doubling"
+ // bit 0=Italian "syntactic doubling" of consoants in the word after a word marked with $double attribute
+ // bit 1=also after a word which ends with a stressed vowel
 #define LOPT_IT_DOUBLING     8
  // max. amplitude for vowel at the end of a clause
 #define LOPT_MAXAMP_EOC      9
- // don't reduce the strongest vowel in a word which is marked 'unstressed'
-#define LOPT_KEEP_UNSTR_VOWEL  10
+ // bit 0=reduce even if phonemes are specified in the **_list file
+ // bit 1=don't reduce the strongest vowel in a word which is marked 'unstressed'
+#define LOPT_REDUCE  10
+ // LANG=cs,sk  combine some prepositions with the following word, if the combination has N or fewer syllables
+ // bits 0-3  N syllables
+ // bit 4=only if the second word has $alt attribute
+ // bit 5=not if the second word is end-of-sentence
+#define LOPT_COMBINE_WORDS 11
+ // change [t] when followed by unstressed vowel
+#define LOPT_REDUCE_T 12
 
 
 typedef struct {
@@ -249,7 +264,11 @@ typedef struct {
 // bit6=light syllable followed by heavy, move secondary stress to the heavy syllable. LANG=Finnish
 // bit8=stress last syllable if it doesn't end in a vowel
 // bit9=stress last syllable if it doesn't end in vowel or "s" or "n"  LANG=Spanish
-// bit12= In 2-syllable words, give the first syllable a secondary stress
+// bit12= In a 2-syllable word, if one has primary stress then give the other secondary stress
+// bit13= If there is only one syllable before the primary stress, give it a secondary stress
+// bit15= Give stress to the first unstressed syllable
+// bit16= Don't diminish consecutive syllables within a word.
+// bit17= "priority" stress reduces other primary stress to "unstressed" not "secondary"
 
 	int stress_flags; 
 	int unstressed_wd1; // stress for $u word of 1 syllable
@@ -258,8 +277,10 @@ typedef struct {
 	unsigned char *length_mods;
 	unsigned char *length_mods0;
 
-	// bits0-2=which numbers routine to use. 0=none, 1=english/german
-	// bit3=  . , for thousands and decimal separator
+#define NUM_ROMAN   0x20000
+	// bits0-1=which numbers routine to use.
+	// bit2=  thousands separator must be space
+	// bit3=  , decimal separator, not .
 	// bit4=use three-and-twenty rather than twenty-three
 	// bit5='and' between tens and units
 	// bit6=add "and" after hundred and thousand
@@ -269,11 +290,18 @@ typedef struct {
 	// bit10=omit "one" before "hundred"
 	// bit11=don't say 19** as nineteen hundred
 	// bit12=allow space as thousands separator (in addition to langopts.thousands_sep)
-	// bit13=(LANG=it) speak post-decimal-point digits as a combined number not as single digits
+	// bits13-15  post-decimal-digits 0=single digits, 1=(LANG=it) 2=(LANG=pl) 3=(LANG=ro)
 	// bit16=dot after number indicates ordinal
-	// bit17=use feminine form of 2 before thousands and millions LANG=ro
-	// bit18=special word for 100,000s LANG=sw
+	// bit17=recognize roman numbers
+
 	int numbers;
+
+	// bits 1-4  use variant form of numbers before thousands,millions,etc.
+	// bit6=(LANG=pl) two forms of plural, M or MA
+	// bit7=(LANG-ru) use MB for 1 thousand, million, etc
+	// bit8=(LANG=sw) special word for 100,000s
+	int numbers2;
+	int max_roman;
 	int thousands_sep;
 	int decimal_sep;
 	int intonation;          // 1=tone language
@@ -284,8 +312,8 @@ typedef struct {
 	char tone_numbers;
 	char ideographs;      // treat as separate words
 	int testing;            // testing options: bit 1= specify stressed syllable in the form:  "outdoor/2"
-	const wchar_t *replace_chars;   // characters to be substitutes
-	const wchar_t *replacement_chars;  // substitutes for replace_chars
+	const unsigned int *replace_chars;      // characters to be substitutes
+	const unsigned int *replacement_chars;  // substitutes for replace_chars
 } LANGUAGE_OPTIONS;
 
 
@@ -335,8 +363,7 @@ public:
 // holds properties of characters: vowel, consonant, etc for pronunciation rules
 	unsigned char letter_bits[256];
 	int letter_bits_offset;
-#define N_LETTER_TYPES 20
-	const wchar_t *letter_groups[N_LETTER_TYPES];
+	const wchar_t *letter_groups[8];
 
 	/* index1=option, index2 by 0=. 1=, 2=?, 3=! 4=none */
 	unsigned char punct_to_tone[4][5];
@@ -352,7 +379,7 @@ private:
 	void SetSpellingStress(char *phonemes, int control);
 	void GetTranslatedPhonemeString(char *phon_out, int n_phon_out);
 	void WriteMnemonic(int *ix, int mnem);
-	void MakePhonemeList(int post_pause, int embedded, int new_sentence);
+	void MakePhonemeList(int post_pause, int new_sentence);
 	int SubstitutePhonemes(PHONEME_LIST2 *plist_out);
 
 	int ReadClause(FILE *f_in, char *buf, unsigned short *charix, int n_buf);
@@ -365,15 +392,17 @@ private:
 	int LookupNum3(int value, char *ph_out, int suppress_null, int thousandplex, int prev_thousands);
 	int LookupThousands(int value, int thousandplex, char *ph_out);
    int TranslateNumber_1(char *word1, char *ph_out, unsigned int *flags, int wflags);
+	int TranslateRoman(char *word, char *ph_out);
 
 	void InitGroups(void);
 	void AppendPhonemes(char *string, int size, const char *ph);
 	char *DecodeRule(const char *group, char *rule);
-	void MatchRule(char *word[], const char *group, char *rule, MatchRecord *match_out, int end_flags);
+	void MatchRule(char *word[], const char *group, char *rule, MatchRecord *match_out, int end_flags, int dict_flags);
 	int TranslateRules(char *p, char *phonemes, int size, char *end_phonemes, int end_flags, int dict_flags);
 	void ApplySpecialAttribute(char *phonemes, int dict_flags);
 
 	int IsLetter(int letter, int group);
+	int IsLetterGroup(char *word, int group);
 
 	void CalcPitches_Tone(int clause_tone);
 
@@ -381,7 +410,7 @@ protected:
 	virtual int Unpronouncable(char *word);
 	virtual void SetWordStress(char *output, unsigned int dictionary_flags, int tonic, int prev_stress);
 	virtual int RemoveEnding(char *word, int end_type, char *word_copy);
-	virtual int TranslateChar(char *ptr, int prev_in, int c, int next_in);
+	virtual int TranslateChar(char *ptr, int prev_in, unsigned int c, unsigned int next_in, int *insert);
    virtual int TranslateNumber(char *word1, char *ph_out, unsigned int *flags, int wflags);
 	virtual int ChangePhonemes(PHONEME_LIST2 *phlist, int n_ph, int index, PHONEME_TAB *ph, CHANGEPH *ch);
 
@@ -400,6 +429,7 @@ protected:
 	
 	unsigned char groups2_count[256];    // number of 2 letter groups for this initial letter
 	unsigned char groups2_start[256];    // index into groups2
+	char *letterGroups[N_LETTER_GROUPS];
 	
 	int n_ph_list2;
 	PHONEME_LIST2 ph_list2[N_PHONEME_LIST];	// first stage of text->phonemes
@@ -428,7 +458,9 @@ protected:
 extern int option_tone1;
 extern int option_tone2;
 extern int option_waveout;
+extern int option_quiet;
 extern int option_phonemes;
+extern int option_phoneme_events;
 extern int option_linelength;     // treat lines shorter than this as end-of-clause
 extern int option_harmonic1;
 extern int option_multibyte;
@@ -470,20 +502,22 @@ extern char *p_textinput;
 extern wchar_t *p_wchar_input;
 extern int ungot_char;
 extern int (* uri_callback)(int, const char *, const char *);
+extern int (* phoneme_callback)(const char *);
 extern void SetLengthMods(Translator *tr, int value);
 
 Translator *SelectTranslator(const char *name);
 int CompileDictionary(const char *dsource, const char *dict_name, FILE *log, char *err_name);
 void LoadConfig(void);
 int PhonemeCode(unsigned int mnem);
-void ChangeWordStress(char *word, int new_stress);
+void ChangeWordStress(Translator *tr, char *word, int new_stress);
 int TransposeAlphabet(char *text, int offset, int min, int max);
 int utf8_in(int *c, char *buf, int backwards);
 int utf8_out(unsigned int c, char *buf);
-int lookupwchar(const short *list,int c);
+int lookupwchar(const unsigned short *list,int c);
 int Eof(void);
 char *strchr_w(const char *s, int c);
 int IsBracket(int c);
+void ResetLetterBits(Translator *tr, int groups);
 void SetLetterBits(Translator *tr, int group, const char *string);
 void SetLetterBitsRange(Translator *tr, int group, int first, int last);
 void SetLetterVowel(Translator *tr, int c);
@@ -493,5 +527,6 @@ void InitText2(void);
 int IsDigit(unsigned int c);
 int IsAlpha(unsigned int c);
 int isspace2(unsigned int c);
+void SetVoiceStack(espeak_VOICE *v);
 
 extern FILE *f_trans;		// for logging
