@@ -1,10 +1,10 @@
 /***************************************************************************
- *   Copyright (C) 2005 by Jonathan Duddington                             *
- *   jonsd@users.sourceforge.net                                           *
+ *   Copyright (C) 2005 to 2007 by Jonathan Duddington                     *
+ *   email: jonsd@users.sourceforge.net                                    *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
- *   the Free Software Foundation; either version 2 of the License, or     *
+ *   the Free Software Foundation; either version 3 of the License, or     *
  *   (at your option) any later version.                                   *
  *                                                                         *
  *   This program is distributed in the hope that it will be useful,       *
@@ -13,12 +13,12 @@
  *   GNU General Public License for more details.                          *
  *                                                                         *
  *   You should have received a copy of the GNU General Public License     *
- *   along with this program; if not, write to the                         *
- *   Free Software Foundation, Inc.,                                       *
- *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
+ *   along with this program; if not, write see:                           *
+ *               <http://www.gnu.org/licenses/>.                           *
  ***************************************************************************/
 
 
+
 typedef struct {
 	char name[40];
 
@@ -36,16 +36,21 @@ typedef struct {
 	int echo_amp;
 	int n_harmonic_peaks;  // highest formant which is formed from adding harmonics
 	int peak_shape;        // alternative shape for formant peaks (0=standard 1=squarer)
+	int voicing;           // 100% = 64, level of formant-synthesized sound
+	int formant_factor;      // adjust nominal formant frequencies by this  because of the voice's pitch (256ths)
 
 	// parameters used by Wavegen
-	int freq[N_PEAKS];    // 100% = 256
-	int height[N_PEAKS];  // 100% = 256
-	int width[N_PEAKS];   // 100% = 256
+	short freq[N_PEAKS];    // 100% = 256
+	short height[N_PEAKS];  // 100% = 256
+	short width[N_PEAKS];   // 100% = 256
 
 	// copies without temporary adjustments from embedded commands
-	int freq2[N_PEAKS];    // 100% = 256
-	int height2[N_PEAKS];  // 100% = 256
-	int width2[N_PEAKS];   // 100% = 256
+	short freq2[N_PEAKS];    // 100% = 256
+	short height2[N_PEAKS];  // 100% = 256
+	short width2[N_PEAKS];   // 100% = 256
+
+	int breath[N_PEAKS];  // amount of breath for each formant. breath[0] indicates whether any are set.
+	int breathw[N_PEAKS];  // width of each breath formant
 
 	// This table provides the opportunity for tone control.
 	// Adjustment of harmonic amplitudes, steps of 8Hz
@@ -62,7 +67,7 @@ extern USHORT voice_pcnt[N_PEAKS+1][3];
 extern voice_t *voice;
 extern int tone_points[10];
 
-voice_t *LoadVoice(char *voice_name, int control);
+voice_t *LoadVoice(const char *voice_name, int control);
 voice_t *LoadVoiceVariant(const char *voice_name, int variant);
 void WavegenSetVoice(voice_t *v);
 void ReadTonePoints(char *string, int *tone_pts);
