@@ -1,10 +1,10 @@
 /***************************************************************************
- *   Copyright (C) 2005 by Jonathan Duddington                             *
- *   jonsd@users.sourceforge.net                                           *
+ *   Copyright (C) 2005 to 2007 by Jonathan Duddington                     *
+ *   email: jonsd@users.sourceforge.net                                    *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
- *   the Free Software Foundation; either version 2 of the License, or     *
+ *   the Free Software Foundation; either version 3 of the License, or     *
  *   (at your option) any later version.                                   *
  *                                                                         *
  *   This program is distributed in the hope that it will be useful,       *
@@ -13,10 +13,10 @@
  *   GNU General Public License for more details.                          *
  *                                                                         *
  *   You should have received a copy of the GNU General Public License     *
- *   along with this program; if not, write to the                         *
- *   Free Software Foundation, Inc.,                                       *
- *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
+ *   along with this program; if not, write see:                           *
+ *               <http://www.gnu.org/licenses/>.                           *
  ***************************************************************************/
+
 #include "StdAfx.h"
 
 #include "stdio.h"
@@ -35,9 +35,9 @@
 #endif
 
 #include "speak_lib.h"
-#include "voice.h"
 #include "phoneme.h"
 #include "synthesize.h"
+#include "voice.h"
 #include "translate.h"
 
 
@@ -47,20 +47,25 @@ MNEM_TAB genders [] = {
 	{"female", 2},
 	{NULL, 0 }};
 
+//int tone_points[10] = {250,140, 1200,110, -1,0, -1,0, -1,0};
+int tone_points[10] = {600,170, 1200,135, 2000,110, 3000,110, -1,0};
+
 // limit the rate of change for each formant number
-static int formant_rate_22050[9] = {50, 104, 165, 230, 220, 220, 220, 220, 220};  // values for 22kHz sample rate
+//static int formant_rate_22050[9] = {50, 104, 165, 230, 220, 220, 220, 220, 220};  // values for 22kHz sample rate
+//static int formant_rate_22050[9] = {240, 180, 180, 180, 180, 180, 180, 180, 180};  // values for 22kHz sample rate
+static int formant_rate_22050[9] = {240, 170, 170, 170, 170, 170, 170, 170, 170};  // values for 22kHz sample rate
 int formant_rate[9];         // values adjusted for actual sample rate
 
 
 
 #define DEFAULT_LANGUAGE_PRIORITY  5
-#define N_VOICES_LIST  100
+#define N_VOICES_LIST  150
 static int n_voices_list = 0;
 static espeak_VOICE *voices_list[N_VOICES_LIST];
 static int len_path_voices;
 
 espeak_VOICE *voice_selected = NULL;
-char voice_name[40];
+espeak_VOICE *first_voice = NULL;
 
 
 
@@ -79,21 +84,26 @@ char voice_name[40];
 #define V_ROUGHNESS  11
 #define V_CLARITY    12
 #define V_TONE       13
+#define V_VOICING    14
+#define V_BREATH     15
+#define V_BREATHW    16
 
 // these override defaults set by the translator
-#define V_WORDGAP    15
-#define V_INTONATION 16
-#define V_STRESSLENGTH  17
-#define V_STRESSAMP  18
-#define V_STRESSADD  19
-#define V_DICTRULES   20
-#define V_STRESSRULE  21
-#define V_CHARSET     22
-#define V_NUMBERS     23
-#define V_OPTION      24
+#define V_WORDGAP    17
+#define V_INTONATION 18
+#define V_STRESSLENGTH  19
+#define V_STRESSAMP  20
+#define V_STRESSADD  21
+#define V_DICTRULES   22
+#define V_STRESSRULE  23
+#define V_CHARSET     24
+#define V_NUMBERS     25
+#define V_OPTION      26
+
+#define V_MBROLA     27
 
 // these need a phoneme table to have been specified
-#define V_REPLACE    25
+#define V_REPLACE    28
 
 
 
@@ -126,8 +136,12 @@ static keywtab_t keyword_tab[] = {
 	{"roughness",  V_ROUGHNESS},
 	{"clarity",    V_CLARITY},
 	{"tone",       V_TONE},
+	{"voicing",    V_VOICING},
+	{"breath",     V_BREATH},
+	{"breathw",    V_BREATHW},
 	{"numbers",    V_NUMBERS},
 	{"option",     V_OPTION},
+	{"mbrola",     V_MBROLA},
 
 	// these just set a value in langopts.param[]
 	{"l_dieresis", 0x100+LOPT_DIERESES},
@@ -150,7 +164,6 @@ const char variants_female[N_VOICE_VARIANTS] = {11,12,13,14,0};
 const char *variant_lists[3] = {variants_either, variants_male, variants_female};
 
 
-int tone_points[10] = {250,140, 1200,110, -1,0, -1,0, -1,0};
 
 void SetToneAdjust(voice_t *voice, int *tone_pts)
 {//==============================================
@@ -205,31 +218,156 @@ void ReadTonePoints(char *string, int *tone_pts)
 }
 
 
+
+
+static espeak_VOICE *ReadVoiceFile(FILE *f_in, const char *fname, const char*leafname)
+{//===================================================================================
+// Read a Voice file, allocate a VOICE_DATA and set data from the
+// file's  language, gender, name  lines
+
+	char linebuf[80];
+	char vname[80];
+	char vgender[80];
+	char vlanguage[80];
+	char languages[300];  // allow space for several alternate language names and priorities
+
+
+	unsigned int len;
+	int langix = 0;
+	int n_languages = 0;
+	char *p;
+	espeak_VOICE *voice_data;
+	int priority;
+	int age;
+	int n_variants = 3;    // default, number of variants of this voice before using another voice
+	int gender;
+
+#ifdef PLATFORM_WINDOWS
+	char fname_buf[sizeof(path_home)+15];
+	if(memcmp(leafname,"mb-",3) == 0)
+	{
+		// check whether the mbrola speech data is present for this voice
+		memcpy(vname,&leafname[3],3);
+		vname[3] = 0;
+		sprintf(fname_buf,"%s/mbrola/%s",path_home,vname);
+
+		if(GetFileLength(fname_buf) <= 0)
+			return(0);
+	}
+#endif
+
+	vname[0] = 0;
+	vgender[0] = 0;
+	age = 0;
+
+	while(fgets(linebuf,sizeof(linebuf),f_in) != NULL)
+	{
+		linebuf[strlen(linebuf)-1] = 0;
+
+		if(memcmp(linebuf,"name",4)==0)
+		{
+			p = &linebuf[4];
+			while(isspace(*p)) p++;
+			strncpy0(vname,p,sizeof(vname));
+		}
+		else
+		if(memcmp(linebuf,"language",8)==0)
+		{
+			priority = DEFAULT_LANGUAGE_PRIORITY;
+			vlanguage[0] = 0;
+
+			sscanf(&linebuf[8],"%s %d",vlanguage,&priority);
+			len = strlen(vlanguage) + 2;
+			// check for space in languages[]
+			if(len < (sizeof(languages)-langix-1))
+			{
+				languages[langix] = priority;
+
+				strcpy(&languages[langix+1],vlanguage);
+				langix += len;
+				n_languages++;
+			}
+		}
+		else
+		if(memcmp(linebuf,"gender",6)==0)
+		{
+			sscanf(&linebuf[6],"%s %d",vgender,&age);
+		}
+		else
+		if(memcmp(linebuf,"variants",8)==0)
+		{
+			sscanf(&linebuf[8],"%d",&n_variants);
+		}
+	}
+	languages[langix++] = 0;
+
+	gender = LookupMnem(genders,vgender);
+
+	if(n_languages == 0)
+	{
+		return(NULL);    // no language lines in the voice file
+	}
+
+	p = (char *)calloc(sizeof(espeak_VOICE) + langix + strlen(fname) + strlen(vname) + 3, 1);
+	voice_data = (espeak_VOICE *)p;
+	p = &p[sizeof(espeak_VOICE)];
+
+	memcpy(p,languages,langix);
+	voice_data->languages = p;
+
+	strcpy(&p[langix],fname);
+	voice_data->identifier = &p[langix];
+	voice_data->name = &p[langix];
+
+	if(vname[0] != 0)
+	{
+		langix += strlen(fname)+1;
+		strcpy(&p[langix],vname);
+		voice_data->name = &p[langix];
+	}
+
+	voice_data->age = age;
+	voice_data->gender = gender;
+	voice_data->variant = 0;
+	voice_data->xx1 = n_variants;
+	return(voice_data);
+}  // end of ReadVoiceFile
+
+
+
+
 void VoiceReset(int tone_only)
 {//===========================
 // Set voice to the default values
+
 	int  pk;
+	static int breath_widths[N_PEAKS] = {0,200,200,400,400,400,600,600,600};
 
 	// default is:  pitch 82,118
 	voice->pitch_base =   0x49000;    // default, 73 << 12;
 	voice->pitch_range =  0x0f30;     // default = 0x1000
+	voice->formant_factor = 256;
 
 	voice->echo_delay = 0;
 	voice->echo_amp = 0;
 	voice->flutter = 64;
 	voice->n_harmonic_peaks = 5;
 	voice->peak_shape = 1;
+	voice->voicing = 64;
 #ifdef PLATFORM_RISCOS
 	voice->roughness = 1;
 #else
 	voice->roughness = 2;
 #endif
 
+	InitBreath();
 	for(pk=0; pk<N_PEAKS; pk++)
 	{
 		voice->freq[pk] = 256;
 		voice->height[pk] = 256;
 		voice->width[pk] = 256;
+		voice->breath[pk] = 0;
+		voice->breathw[pk] = breath_widths[pk];  // default breath formant woidths
 
 		// adjust formant smoothing depending on sample rate
 		formant_rate[pk] = (formant_rate_22050[pk] * 22050)/samplerate;
@@ -250,6 +388,8 @@ SetToneAdjust(voice,tone_points);
 	{
 		n_replace_phonemes = 0;
 		option_tone1 = 0;
+		option_quiet = 0;
+		LoadMbrolaTable(NULL,NULL);
 	}
 }  // end of VoiceReset
 
@@ -281,11 +421,17 @@ static void VoiceFormant(char *p)
 
 static voice_t *VoiceLookup(char *voicename)
 {//=========================================
+// Keep a cache of previously used voices.
 	int ix;
 	voice_t *v;
 
 	for(ix=0; ix < N_VOICES; ix++)
 	{
+		if((ix < n_voices_tab) && (strcmp(voices_tab[ix]->name,voicename)==0))
+		{
+			return(voices_tab[ix]);   // found the entry for the specified voice name
+		}
+
 		if(ix == n_voices_tab)
 		{
 			// found a free slot
@@ -296,13 +442,8 @@ static voice_t *VoiceLookup(char *voicename)
 			strncpy0(v->name,voicename,sizeof(v->name));
 			return(v);
 		}
-		else
-		if(strcmp(voices_tab[ix]->name,voicename)==0)
-		{
-			return(voices_tab[ix]);   // found the entry for the specified voice name
-		}
 	}
-	return(NULL);  // table is full
+	return(voices_tab[0]);  // table is full, reuse the first entry
 }  // end of VoiceLookup
 
 
@@ -340,11 +481,12 @@ static int Read8Numbers(char *data_in,int *data)
 }
 
 
-voice_t *LoadVoice(char *vname, int control)
+voice_t *LoadVoice(const char *vname, int control)
 {//==========================================
 // control, bit 0  1= no_default
 //          bit 1  1 = change tone only, not language
 //          bit 2  1 = don't report error on LoadDictionary
+//          bit 4  1 = vname = full path
 
 	FILE *f_voice = NULL;
 	keywtab_t *k;
@@ -373,6 +515,7 @@ voice_t *LoadVoice(char *vname, int control)
 	char phonemes_name[40];
 	char *language_type;
 	char buf[200];
+	char path_voices[sizeof(path_home)+12];
 	char langname[4];
 
 	int stress_amps[8];
@@ -386,15 +529,25 @@ voice_t *LoadVoice(char *vname, int control)
 	if(voicename[0]==0)
 		strcpy(voicename,"default");
 
-	sprintf(buf,"%s%cvoices%c%s",path_home,PATHSEP,PATHSEP,voicename);
-
-	if(GetFileLength(buf) <= 0)
+	if(control & 0x10)
 	{
-		// look for the voice in a sub-directory of the language name
-		langname[0] = voicename[0];
-		langname[1] = voicename[1];
-		langname[2] = 0;
-		sprintf(buf,"%s%cvoices%c%s%c%s",path_home,PATHSEP,PATHSEP,langname,PATHSEP,voicename);
+		strcpy(buf,vname);
+		if(GetFileLength(buf) <= 0)
+			return(NULL);
+	}
+	else
+	{
+		sprintf(path_voices,"%s%cvoices%c",path_home,PATHSEP,PATHSEP);
+		sprintf(buf,"%s%s",path_voices,voicename);
+
+		if(GetFileLength(buf) <= 0)
+		{
+			// look for the voice in a sub-directory of the language name
+			langname[0] = voicename[0];
+			langname[1] = voicename[1];
+			langname[2] = 0;
+			sprintf(buf,"%s%s%c%s",path_voices,langname,PATHSEP,voicename);
+		}
 	}
 
 	f_voice = fopen(buf,"r");
@@ -409,6 +562,13 @@ voice_t *LoadVoice(char *vname, int control)
 			language_type = voicename;
 	}
 
+	if((first_voice == NULL) && (f_voice != NULL))
+	{
+		first_voice = ReadVoiceFile(f_voice,buf+strlen(path_voices),voicename);
+		rewind(f_voice);
+	}
+
+
 	if(!tone_only && (translator != NULL))
 	{
 		delete translator;
@@ -420,8 +580,11 @@ voice_t *LoadVoice(char *vname, int control)
 	strcpy(phonemes_name,language_type);
 
 
-	if((v = VoiceLookup(voicename)) != NULL)
-		voice = v;
+	if(!tone_only)
+	{
+		if((v = VoiceLookup(voicename)) != NULL)
+			voice = v;
+	}
 	VoiceReset(tone_only);
 
 	if(!tone_only)
@@ -430,14 +593,15 @@ voice_t *LoadVoice(char *vname, int control)
 
 	while((f_voice != NULL) && (fgets(buf,sizeof(buf),f_voice) != NULL))
 	{
+		if((p = strstr(buf,"//")) != NULL)
+			*p = 0;
+
+		// isolate the attribute name
 		for(p=buf; (*p != 0) && !isspace(*p); p++);
 		*p++ = 0;
 
 		if(buf[0] == 0) continue;
 
-		if((buf[0]=='/') && (buf[1]=='/'))
-			continue;   // skip comment line
-
 		key = 0;
 		for(k=keyword_tab; k->mnem != NULL; k++)
 		{
@@ -456,6 +620,9 @@ voice_t *LoadVoice(char *vname, int control)
 				break;
 
 			sscanf(p,"%s",language_name);
+			if(strcmp(language_name,"variant")==0)
+				break;
+
 			language_type = strtok(language_name,"-");
 			language_set = 1;
 			strcpy(translator_name,language_type);
@@ -499,10 +666,15 @@ voice_t *LoadVoice(char *vname, int control)
 			break;
 
 		case V_PITCH:
-			// default is  pitch 82 118
-			n = sscanf(p,"%d %d",&pitch1,&pitch2);
-			voice->pitch_base = (pitch1 - 9) << 12;
-			voice->pitch_range = (pitch2 - pitch1) * 108;
+			{
+				double factor;
+				// default is  pitch 82 118
+				n = sscanf(p,"%d %d",&pitch1,&pitch2);
+				voice->pitch_base = (pitch1 - 9) << 12;
+				voice->pitch_range = (pitch2 - pitch1) * 108;
+				factor = double(pitch1 - 82)/82;
+				voice->formant_factor = (int)((1+factor/4) * 256);  // nominal formant shift for a different voice pitch
+			}
 			break;
 
 		case V_STRESSLENGTH:   // stressLength
@@ -526,8 +698,11 @@ voice_t *LoadVoice(char *vname, int control)
 			{
 				while(isspace(*p)) p++;
 				n = -1;
-				if((n = atoi(p++)) >= 0)
+				if((n = atoi(p)) > 0)
+				{
+					p++;
 					conditional_rules |= (1 << n);
+				}
 				while(isalnum(*p)) p++;
 			}
 			break;
@@ -607,6 +782,34 @@ voice_t *LoadVoice(char *vname, int control)
 			}
 			break;
 
+		case V_VOICING:
+			if(sscanf(p,"%d",&value)==1)
+				voice->voicing = (value * 64)/100;
+			break;
+
+		case V_BREATH:
+				voice->breath[0] = Read8Numbers(p,&voice->breath[1]);
+				for(ix=1; ix<8; ix++)
+				{
+					if(ix % 2)
+						voice->breath[ix] = -voice->breath[ix];
+				}
+			break;
+
+		case V_BREATHW:
+				voice->breathw[0] = Read8Numbers(p,&voice->breathw[1]);
+			break;
+
+		case V_MBROLA:
+			{
+				char name[40];
+				char phtrans[40];
+				phtrans[0] = 0;
+				sscanf(p,"%s %s",name,phtrans);
+				LoadMbrolaTable(name,phtrans);
+			}
+			break;
+
 		default:
 			if((key & 0xff00) == 0x100)
 			{
@@ -666,7 +869,6 @@ voice_t *LoadVoice(char *vname, int control)
 	if(!tone_only)
 	{
 		translator = new_translator;
-		strcpy(voice_name,voicename);
 	}
 
 	// relative lengths of different stress syllables
@@ -688,24 +890,63 @@ voice_t *LoadVoice(char *vname, int control)
 }  //  end of LoadVoice
 
 
-voice_t *LoadVoiceVariant(const char *vname, int variant)
-{//======================================================
-	voice_t *v;
-	char *p;
-	char buf[40];
+char *ExtractVoiceVariantName(char *vname, int variant_num)
+{//========================================================
+// Remove any voice variant suffix (name or number) from a voice name
+// Returns the voice variant name
 
-	strcpy(buf,vname);
-	if((p = strchr(buf,'+')) != NULL)
+	char *p;
+	static char variant_name[20];
+
+	variant_name[0] = 0;
+
+	if((p = strchr(vname,'+')) != NULL)
 	{
 		// The voice name has a +variant suffix
-		*p = 0;
-		variant = atoi(p+1);
+		*p++ = 0;   // delete the suffix from the voice name
+		if(isdigit(*p))
+		{
+			variant_num = atoi(p);  // variant number
+		}
+		else
+		{
+			// voice variant name, not number
+			strcpy(variant_name,"!v/");
+			strncpy0(&variant_name[3],p,sizeof(variant_name)-3);
+		}	
 	}
-	v = LoadVoice(buf,0);
-	if((v != NULL) && (variant > 0))
+	
+	if(variant_num > 0)
 	{
-		sprintf(buf,"!variant%d",variant);
-		LoadVoice(buf,2);
+		if(variant_num < 10)
+			sprintf(variant_name,"!v/m%d",variant_num);  // male
+		else
+			sprintf(variant_name,"!v/f%d",variant_num-10);  // female
+	}
+
+	return(variant_name);
+}  //  end of ExtractVoiceVariantName
+
+
+
+voice_t *LoadVoiceVariant(const char *vname, int variant_num)
+{//==========================================================
+// Load a voice file.
+// Also apply a voice variant if specified by "variant", or by "+number" or "+name" in the "vname"
+
+	voice_t *v;
+	char *variant_name;
+	char buf[60];
+
+	strncpy0(buf,vname,sizeof(buf));
+	variant_name = ExtractVoiceVariantName(buf,variant_num);
+
+	if((v = LoadVoice(buf,0)) == NULL)
+		return(NULL);
+
+	if(variant_name[0] != 0)
+	{
+		v = LoadVoice(variant_name,2);
 	}
 	return(v);
 }
@@ -735,7 +976,7 @@ static int __cdecl VoiceScoreSorter(const void *p1, const void *p2)
 
 	if((ix = v2->score - v1->score) != 0)
 		return(ix);
-	return(strcmp(v2->name,v1->name));
+	return(strcmp(v1->name,v2->name));
 }
 
 
@@ -756,12 +997,18 @@ static int ScoreVoice(espeak_VOICE *voice_spec, int spec_n_parts, int spec_lang_
 
 	p = voice->languages;  // list of languages+dialects for which this voice is suitable
 
-	if(voice_spec->languages == NULL)
+	if(spec_n_parts == 0)
 	{
 		score = 100;
 	}
 	else
 	{
+		if((*p == 0) && (strcmp(voice_spec->languages,"variants")==0))
+		{
+			// match on a voice with no languages if the required language is "variants"
+			score = 100;
+		}
+
 		// compare the required language with each of the languages of this voice
 		while(*p != 0)
 		{
@@ -816,10 +1063,18 @@ static int ScoreVoice(espeak_VOICE *voice_spec, int spec_n_parts, int spec_lang_
 	if(score == 0)
 		return(0);
 
-	if((voice_spec->name != NULL) && (strcmp(voice_spec->name,voice->name)==0))
+	if(voice_spec->name != NULL)
 	{
-		// match on voice name
-		score += 400;
+		if(strcmp(voice_spec->name,voice->name)==0)
+		{
+			// match on voice name
+			score += 500;
+		}
+		else
+		if(strcmp(voice_spec->name,voice->identifier)==0)
+		{
+			score += 400;
+		}
 	}
 
 	if(((voice_spec->gender == 1) || (voice_spec->gender == 2)) &&
@@ -861,19 +1116,21 @@ static int ScoreVoice(espeak_VOICE *voice_spec, int spec_n_parts, int spec_lang_
 }  // end of ScoreVoice
 
 
-static int SetVoiceScores(espeak_VOICE *voice_select, espeak_VOICE **voices)
-{//=========================================================================
+static int SetVoiceScores(espeak_VOICE *voice_select, espeak_VOICE **voices, int control)
+{//======================================================================================
+// control: bit0=1  include mbrola voices
 	int ix;
 	int score;
 	int nv;           // number of candidates
-	int n_parts=1;
+	int n_parts=0;
 	int lang_len=0;
 	char *p;
 	espeak_VOICE *vp;
 
 	// count number of parts in the specified language
-	if(voice_select->languages != NULL)
+	if((voice_select->languages != NULL) && (voice_select->languages[0] != 0))
 	{
+		n_parts = 1;
 		lang_len = strlen(voice_select->languages);
 		for(p = voice_select->languages; *p != 0; p++)
 		{
@@ -881,14 +1138,17 @@ static int SetVoiceScores(espeak_VOICE *voice_select, espeak_VOICE **voices)
 				n_parts++;
 		}
 	}
-
 	// select those voices which match the specified language
 	nv = 0;
 	for(ix=0; ix<n_voices_list; ix++)
 	{
+		vp = voices_list[ix];
+		if(((control & 1) == 0) && (memcmp(vp->identifier,"mb/",3) == 0))
+			continue;
+
 		if((score = ScoreVoice(voice_select, n_parts, lang_len, voices_list[ix])) > 0)
 		{
-			voices[nv++] = vp = voices_list[ix];
+			voices[nv++] = vp;
 			vp->score = score;
 		}
 	}
@@ -906,126 +1166,6 @@ static int SetVoiceScores(espeak_VOICE *voice_select, espeak_VOICE **voices)
 
 
 
-static espeak_VOICE *ReadVoiceFile(FILE *f_in, const char *fname, const char*leafname)
-{//===================================================================================
-// Read a Voice file, allocate a VOICE_DATA and set data from the
-// file's  language, gender, name  lines
-
-	char linebuf[80];
-	char vname[80];
-	char vgender[80];
-	char vlanguage[80];
-	char languages[300];  // allow space for several alternate language names and priorities
-
-
-	unsigned int len;
-	int langix = 0;
-	int n_languages = 0;
-	char *p;
-	espeak_VOICE *voice_data;
-	int priority;
-	int age;
-	int n_variants = 3;    // default, number of variants of this voice before using another voice
-	int gender;
-
-	vname[0] = 0;
-	vgender[0] = 0;
-	age = 0;
-
-	while(fgets(linebuf,sizeof(linebuf),f_in) != NULL)
-	{
-		linebuf[strlen(linebuf)-1] = 0;
-
-		if(memcmp(linebuf,"name",4)==0)
-		{
-			p = &linebuf[4];
-			while(isspace(*p)) p++;
-			strncpy0(vname,p,sizeof(vname));
-		}
-		else
-		if(memcmp(linebuf,"language",8)==0)
-		{
-			priority = DEFAULT_LANGUAGE_PRIORITY;
-			vlanguage[0] = 0;
-
-			sscanf(&linebuf[8],"%s %d",vlanguage,&priority);
-			len = strlen(vlanguage) + 2;
-			// check for space in languages[]
-			if(len < (sizeof(languages)-langix-1))
-			{
-				languages[langix] = priority;
-
-				strcpy(&languages[langix+1],vlanguage);
-				langix += len;
-				n_languages++;
-			}
-		}
-		else
-		if(memcmp(linebuf,"gender",6)==0)
-		{
-			sscanf(&linebuf[6],"%s %d",vgender,&age);
-		}
-		else
-		if(memcmp(linebuf,"variants",8)==0)
-		{
-			sscanf(&linebuf[8],"%d",&n_variants);
-		}
-	}
-	languages[langix++] = 0;
-
-	gender = LookupMnem(genders,vgender);
-
-	if(n_languages == 0)
-	{
-#ifdef deleted
-// Read voice variant files
-// Don't use this, instead preset the variants_* arrays
-
-		// no language is specified, this voice file only affects the voice characteristics
-		if(memcmp(leafname,"!variant",8)==0)
-		{
-			if(((variant = atoi(&leafname[8])) > 0) && (strlen(variants_either) < N_VOICE_VARIANTS))
-			{
-				char string[2];
-				string[0] = variant;
-				string[1] = 0;
-				strcat(variants_either,string);
-				if(gender == 1)
-					strcat(variants_male,string);
-				if(gender == 2)
-					strcat(variants_female,string);
-			}
-		}
-#endif
-		return(NULL);    // no language lines in the voice file
-	}
-
-	p = (char *)calloc(sizeof(espeak_VOICE) + langix + strlen(fname) + strlen(vname) + 3, 1);
-	voice_data = (espeak_VOICE *)p;
-	p = &p[sizeof(espeak_VOICE)];
-
-	memcpy(p,languages,langix);
-	voice_data->languages = p;
-
-	strcpy(&p[langix],fname);
-	voice_data->identifier = &p[langix];
-	voice_data->name = &p[langix];
-
-	if(vname[0] != 0)
-	{
-		langix += strlen(fname)+1;
-		strcpy(&p[langix],vname);
-		voice_data->name = &p[langix];
-	}
-
-	voice_data->age = age;
-	voice_data->gender = gender;
-	voice_data->variant = 0;
-	voice_data->xx1 = n_variants;
-	return(voice_data);
-}  // end of ReadVoiceFile
-
-
 static espeak_VOICE *SelectVoiceByName(espeak_VOICE **voices, const char *name)
 {//============================================================================
 	int ix;
@@ -1034,7 +1174,7 @@ static espeak_VOICE *SelectVoiceByName(espeak_VOICE **voices, const char *name)
 	int match_name = -1;
 	char *id;
 	int last_part_len;
-	char last_part[20];
+	char last_part[41];
 
 	sprintf(last_part,"%c%s",PATHSEP,name);
 	last_part_len = strlen(last_part);
@@ -1077,6 +1217,7 @@ static espeak_VOICE *SelectVoiceByName(espeak_VOICE **voices, const char *name)
 espeak_VOICE *SelectVoice(espeak_VOICE *voice_select, int *variant)
 {//================================================================
 // Returns a path within espeak-voices, with a possible +variant suffix
+// variant is an output-only parameter
 	int nv;           // number of candidates
 	int ix, ix2;
 	int j;
@@ -1088,16 +1229,51 @@ espeak_VOICE *SelectVoice(espeak_VOICE *voice_select, int *variant)
 	const char *p, *p_start;
 	espeak_VOICE *vp = NULL;
 	espeak_VOICE *vp2;
+	espeak_VOICE voice_select2;
 	espeak_VOICE *voices[N_VOICES_LIST]; // list of candidates
 	espeak_VOICE *voices2[N_VOICES_LIST+N_VOICE_VARIANTS];
+	static espeak_VOICE voice_variants[N_VOICE_VARIANTS];
 
-	espeak_VOICE voice_variants[N_VOICE_VARIANTS];
+	memcpy(&voice_select2,voice_select,sizeof(voice_select2));
 
 	if(n_voices_list == 0)
 		espeak_ListVoices(NULL);   // create the voices list
 
+	if((voice_select2.languages == NULL) || (voice_select2.languages[0] == 0))
+	{
+		// no language is specified. Get language from the named voice
+		int var;
+		char *p2;
+		static char buf[60];
+	
+		if(voice_select2.name == NULL)
+		{
+			if((voice_select2.name = voice_select2.identifier) == NULL)
+				voice_select2.name = "default";
+		}
+	
+		strncpy0(buf,voice_select2.name,sizeof(buf));
+		if((p2 = strchr(buf,'+')) != NULL)
+		{
+			// remove the voice variant suffix, from eg. en+3
+			*p2 = 0;
+			var = atoi(p2+1);
+		}
+		vp = SelectVoiceByName(voices_list,buf);
+		if(vp != NULL)
+		{
+			voice_select2.languages = &(vp->languages[1]);
+
+			if((voice_select2.gender==0) && (voice_select2.age==0) && (voice_select2.variant==0))
+			{
+				*variant = var;
+				return(vp);
+			}
+		}
+	}
+
 	// select and sort voices for the required language
-	nv = SetVoiceScores(voice_select,voices);
+	nv = SetVoiceScores(&voice_select2,voices,0);
 
 	if(nv == 0)
 	{
@@ -1107,14 +1283,14 @@ espeak_VOICE *SelectVoice(espeak_VOICE *voice_select, int *variant)
 	}
 
 	gender = 0;
-	if((voice_select->gender == 2) || ((voice_select->age > 0) && (voice_select->age < 13)))
+	if((voice_select2.gender == 2) || ((voice_select2.age > 0) && (voice_select2.age < 13)))
 		gender = 2;
 	else
-	if(voice_select->gender == 1)
+	if(voice_select2.gender == 1)
 		gender = 1;
 
 #define AGE_OLD  60
-	if(voice_select->age < AGE_OLD)
+	if(voice_select2.age < AGE_OLD)
 		aged = 0;
 
 	p = p_start = variant_lists[gender];
@@ -1167,7 +1343,7 @@ espeak_VOICE *SelectVoice(espeak_VOICE *voice_select, int *variant)
 	}
 
 	// index the sorted list by the required variant number
-	vp = voices2[voice_select->variant % ix2];
+	vp = voices2[voice_select2.variant % ix2];
 	*variant = vp->variant;
 	return(vp);
 }  //  end of SelectVoice
@@ -1179,7 +1355,7 @@ void GetVoices(const char *path)
 	FILE *f_voice;
 	espeak_VOICE *voice_data;
 	int ftype;
-	char fname[80];
+	char fname[sizeof(path_home)+100];
 
 #ifdef PLATFORM_RISCOS
 #else
@@ -1188,6 +1364,7 @@ void GetVoices(const char *path)
    HANDLE hFind = INVALID_HANDLE_VALUE;
    DWORD dwError;
 
+#undef UNICODE         // we need FindFirstFileA() which takes an 8-bit c-string
 	sprintf(fname,"%s\\*",path);
 	hFind = FindFirstFile(fname, &FindFileData);
 	if(hFind == INVALID_HANDLE_VALUE)
@@ -1204,7 +1381,7 @@ void GetVoices(const char *path)
 			GetVoices(fname);
 		}
 		else
-		if((ftype > 0) && (strcmp(FindFileData.cFileName,"default") != 0))
+		if(ftype > 0)
 		{
 			// a regular line, add it to the voices list	
 			if((f_voice = fopen(fname,"r")) == NULL)
@@ -1244,7 +1421,7 @@ void GetVoices(const char *path)
 			GetVoices(fname);
 		}
 		else
-		if((ftype > 0) && (strcmp(ent->d_name,"default") != 0))
+		if(ftype > 0)
 		{
 			// a regular line, add it to the voices list	
 			if((f_voice = fopen(fname,"r")) == NULL)
@@ -1270,16 +1447,31 @@ void GetVoices(const char *path)
 espeak_ERROR SetVoiceByName(const char *name)
 {//=========================================
 	espeak_VOICE *v;
-	int variant=0;
-	char *p;
-	char buf[80];
+	espeak_VOICE voice_selector;
+	char *variant_name;
+	static char buf[60];
 
 	strncpy0(buf,name,sizeof(buf));
-	if((p = strchr(buf,'+')) != NULL)
+	variant_name = ExtractVoiceVariantName(buf,0);
+
+	memset(&voice_selector,0,sizeof(voice_selector));
+	voice_selector.name = buf;
+
+	// first check for a voice with this filename
+	// This may avoid the need to call espeak_ListVoices().
+
+	if((first_voice == NULL) && (LoadVoice(buf,1) != NULL))
 	{
-		// remove the voice variant suffix, from eg. en+3
-		*p = 0;
-		variant = atoi(p+1);
+		voice_selected = first_voice;
+
+		if(variant_name[0] != 0)
+		{
+			LoadVoice(variant_name,2);
+		}
+
+		WavegenSetVoice(voice);
+		SetVoiceStack(&voice_selector);
+		return(EE_OK);
 	}
 
 	if(n_voices_list == 0)
@@ -1287,15 +1479,21 @@ espeak_ERROR SetVoiceByName(const char *name)
 
 	if((v = SelectVoiceByName(voices_list,buf)) != NULL)
 	{
-		if(LoadVoiceVariant(v->identifier,variant) != NULL)
+		if(LoadVoice(v->identifier,0) != NULL)
 		{
 			voice_selected = v;
+
+			if(variant_name[0] != 0)
+			{
+				LoadVoice(variant_name,2);
+			}
 			WavegenSetVoice(voice);
+			SetVoiceStack(&voice_selector);
 			return(EE_OK);
 		}
 	}
 	return(EE_INTERNAL_ERROR);   // voice name not found
-}  // end of espeak_SetVoiceByName
+}  // end of SetVoiceByName
 
 
 
@@ -1307,8 +1505,10 @@ espeak_ERROR SetVoiceByProperties(espeak_VOICE *voice_selector)
 
 	LoadVoiceVariant(voice_selected->identifier,variant);
 	WavegenSetVoice(voice);
+	SetVoiceStack(voice_selector);
+
 	return(EE_OK);
-}  //  end of espeak_SetVoiceByProperties
+}  //  end of SetVoiceByProperties
 
 
 
@@ -1319,17 +1519,19 @@ espeak_ERROR SetVoiceByProperties(espeak_VOICE *voice_selector)
 #pragma GCC visibility push(default)
 
 
-extern "C" const espeak_VOICE **espeak_ListVoices(espeak_VOICE *voice_spec)
+ESPEAK_API const espeak_VOICE **espeak_ListVoices(espeak_VOICE *voice_spec)
 {//========================================================================
 #ifndef PLATFORM_RISCOS
 	int ix;
+	int j;
+	espeak_VOICE *v;
 	static espeak_VOICE *voices[N_VOICES_LIST];
 	char selected_voice_id[80];
-	char path_voices[80];
+	char path_voices[sizeof(path_home)+12];
 
 	// free previous voice list data
-	if(voice_selected != NULL)
-		strcpy(selected_voice_id,voice_selected->identifier);
+	if((voice_selected != NULL) && (voice_selected->identifier != NULL))
+		strncpy0(selected_voice_id,voice_selected->identifier,sizeof(selected_voice_id));
 	else
 		selected_voice_id[0] = 0;
 	voice_selected = NULL;
@@ -1367,17 +1569,29 @@ extern "C" const espeak_VOICE **espeak_ListVoices(espeak_VOICE *voice_spec)
 	if(voice_spec)
 	{
 		// select the voices which match the voice_spec, and sort them by preference
-		SetVoiceScores(voice_spec,voices);
-		return((const espeak_VOICE **)voices);
+		SetVoiceScores(voice_spec,voices,1);
 	}
-
+	else
+	{
+		// omit variant voices
+		j = 0;
+		for(ix=0; (v = voices_list[ix]) != NULL; ix++)
+		{
+			if((v->languages[0] != 0) && (strcmp(&v->languages[1],"variant") != 0))
+			{
+				voices[j++] = v;
+			}
+		}
+		voices[j] = NULL;
+	}
+	return((const espeak_VOICE **)voices);
 #endif
 	return((const espeak_VOICE **)voices_list);
 }  //  end of espeak_ListVoices
 
 
 
-extern "C" espeak_VOICE *espeak_GetCurrentVoice(void)
+ESPEAK_API espeak_VOICE *espeak_GetCurrentVoice(void)
 {//==================================================
 	return(voice_selected);
 }
