@@ -4,7 +4,7 @@
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
- *   the Free Software Foundation; either version 2 of the License, or     *
+ *   the Free Software Foundation; either version 3 of the License, or     *
  *   (at your option) any later version.                                   *
  *                                                                         *
  *   This program is distributed in the hope that it will be useful,       *
@@ -55,7 +55,7 @@ enum {ONE_BILLION=1000000000};
 
 static t_wave_callback* my_callback_is_output_enabled=NULL;
 
-#define N_WAV_BUF   15
+#define N_WAV_BUF   10
 #define SAMPLE_RATE 22050
 #define FRAMES_PER_BUFFER 512
 #define BUFFER_LENGTH (SAMPLE_RATE*2*sizeof(uint16_t))
@@ -64,10 +64,11 @@ static char myBuffer[BUFFER_LENGTH];
 static char* myRead=NULL; 
 static char* myWrite=NULL; 
 static int out_channels=1;
+static int my_stream_could_start=0;
 
+static int mInCallbackFinishedState = false;
 #if (USE_PORTAUDIO == 18)
 static PortAudioStream *pa_stream=NULL;
-static int mInCallbackFinishedState = false;
 #endif
 #if (USE_PORTAUDIO == 19)
 static struct PaStreamParameters myOutputParameters;
@@ -99,11 +100,11 @@ static void init_buffer()
   SHOW("init_buffer > myRead=0x%x, myWrite=0x%x, BUFFER_LENGTH=0x%x, myReadPosition = myWritePosition = 0\n", (int)myRead, (int)myWrite, BUFFER_LENGTH);
 }
 
-static int get_used_mem()
+static unsigned int get_used_mem()
 {
   char* aRead = myRead;
   char* aWrite = myWrite;
-  int used = 0;
+  unsigned int used = 0;
 
   assert ((aRead >= myBuffer)
 	  && (aRead <= myBuffer + BUFFER_LENGTH)
@@ -123,6 +124,34 @@ static int get_used_mem()
   return used;
 }
 
+//>
+//<start stream
+
+static void start_stream()
+{
+  PaError err;
+  SHOW_TIME("start_stream");
+
+  my_stream_could_start=0;  
+  mInCallbackFinishedState = false;
+
+  err = Pa_StartStream(pa_stream);
+  SHOW("start_stream > Pa_StartStream=%d (%s)\n", err, Pa_GetErrorText(err));
+  
+#if USE_PORTAUDIO == 19
+  if(err == paStreamIsNotStopped)
+    {
+      SHOW_TIME("start_stream > restart stream (begin)");
+      // not sure why we need this, but PA v19 seems to need it
+      err = Pa_StopStream(pa_stream);
+      SHOW("start_stream > Pa_StopStream=%d (%s)\n", err, Pa_GetErrorText(err));
+      err = Pa_StartStream(pa_stream);
+      SHOW("start_stream > Pa_StartStream=%d (%s)\n", err, Pa_GetErrorText(err));
+      SHOW_TIME("start_stream > restart stream (end)");
+    }
+#endif
+}
+
 //>
 //<pa_callback
 
@@ -157,9 +186,7 @@ static int pa_callback(void *inputBuffer, void *outputBuffer,
 	{
 	  SHOW_TIME("pa_callback > underflow");
 	  aResult=1; // paComplete;
-#if (USE_PORTAUDIO == 18)
 	  mInCallbackFinishedState = true;
-#endif
 	  size_t aUsedMem=0;
 	  aUsedMem = (size_t)(aWrite - myRead);
 	  if (aUsedMem)
@@ -267,6 +294,22 @@ static int pa_callback(void *inputBuffer, void *outputBuffer,
 }  //  end of WaveCallBack
 
 //>
+
+
+void wave_flush(void* theHandler)
+{
+  ENTER("wave_flush");
+
+  if (my_stream_could_start)
+    {
+//       #define buf 1024
+//       static char a_buffer[buf*2];
+//       memset(a_buffer,0,buf*2);
+//       wave_write(theHandler, a_buffer, buf*2);
+      start_stream();
+    }
+}
+
 //<wave_open_sound
 
 static int wave_open_sound()
@@ -393,6 +436,7 @@ static int wave_open_sound()
 
 	  //	  err = Pa_OpenDefaultStream(&pa_stream,0,2,paInt16,(double)SAMPLE_RATE,FRAMES_PER_BUFFER,pa_callback,(void *)userdata);
 	}
+      mInCallbackFinishedState = false;
 #endif
     }
 
@@ -575,9 +619,7 @@ void wave_init()
   PaError err;
 
   pa_stream = NULL;
-#if (USE_PORTAUDIO == 18)
   mInCallbackFinishedState = false;
-#endif
   init_buffer();
 
   // PortAudio sound output library
@@ -643,7 +685,7 @@ size_t wave_write(void* theHandler, char* theMono16BitsWaveBuffer, size_t theSiz
   ENTER("wave_write");
   size_t bytes_written = 0;
   size_t bytes_to_write = (out_channels==1) ? theSize : theSize*2;
-  int aNewStream = 0;
+  my_stream_could_start = 0;
 
   if(pa_stream == NULL)
     {
@@ -653,11 +695,11 @@ size_t wave_write(void* theHandler, char* theMono16BitsWaveBuffer, size_t theSiz
 	  SHOW_TIME("wave_write > wave_open_sound fails!");
 	  return 0;
 	}
-      aNewStream=1;
+      my_stream_could_start=1;
     }
   else if (!wave_is_busy(NULL))
     {
-      aNewStream = 1;
+      my_stream_could_start = 1;
     }
   assert(BUFFER_LENGTH >= bytes_to_write);
 
@@ -733,33 +775,9 @@ size_t wave_write(void* theHandler, char* theMono16BitsWaveBuffer, size_t theSiz
   bytes_written = theSize;
   myWritePosition += theSize/sizeof(uint16_t); // add number of samples
 
-  if (aNewStream && (get_used_mem() >= out_channels * FRAMES_PER_BUFFER))
+  if (my_stream_could_start && (get_used_mem() >= out_channels * sizeof(uint16_t) * FRAMES_PER_BUFFER))
     {
-      SHOW_TIME("wave_write > start stream");
-      
-#if (USE_PORTAUDIO == 18)
-      mInCallbackFinishedState = false;
-#endif
-
-      PaError err = Pa_StartStream(pa_stream);
-      SHOW("wave_write > Pa_StartStream=%d (%s)\n", err, Pa_GetErrorText(err));
-      
-#if USE_PORTAUDIO == 19
-      if(err == paStreamIsNotStopped)
-	{
-	  SHOW_TIME("wave_write > restart stream (begin)");
-	  // not sure why we need this, but PA v19 seems to need it
-	  err = Pa_StopStream(pa_stream);
-	  SHOW("wave_write > Pa_StopStream=%d (%s)\n", err, Pa_GetErrorText(err));
-	  err = Pa_StartStream(pa_stream);
-	  SHOW("wave_write > Pa_StartStream=%d (%s)\n", err, Pa_GetErrorText(err));
-	  SHOW_TIME("wave_write > restart stream (end)");
-	}
-#endif
-//       if(err != paNoError)
-// 	{
-// 	  samples_written = 0;
-// 	}
+      start_stream();
     }
 
   SHOW_TIME("wave_write > LEAVE");
@@ -848,6 +866,7 @@ int wave_close(void* theHandler)
       Pa_CloseStream( pa_stream );
       SHOW_TIME("wave_close > Pa_CloseStream (end)");
       pa_stream = NULL;
+      mInCallbackFinishedState = false;
     }
 #else
   if (pa_stream)
@@ -924,7 +943,8 @@ int wave_is_busy(void* theHandler)
       active = Pa_StreamActive(pa_stream) 
 	&& (mInCallbackFinishedState == false);
 #else
-      active = Pa_IsStreamActive(pa_stream);
+      active = Pa_IsStreamActive(pa_stream)
+	&& (mInCallbackFinishedState == false);
 #endif
     }
   
@@ -1000,6 +1020,7 @@ void *wave_test_get_write_buffer()
 #else
 // notdef USE_PORTAUDIO
 
+void wave_flush(void* theHandler) {}
 void wave_init() {}
 void* wave_open(char* the_api) {return (void *)1;}
 size_t wave_write(void* theHandler, char* theMono16BitsWaveBuffer, size_t theSize) {return theSize;}
@@ -1008,10 +1029,17 @@ int wave_is_busy(void* theHandler) {return 0;}
 void wave_terminate() {}
 uint32_t wave_get_read_position(void* theHandler) {return 0;}
 uint32_t wave_get_write_position(void* theHandler) {return 0;}
-uint32_t wave_get_remaining_time(uint32_t sample) {return 0;}
+void wave_flush(void* theHandler) {}
 typedef int (t_wave_callback)(void);
 void wave_set_callback_is_output_enabled(t_wave_callback* cb) {}
-extern uint32_t wave_test_get_write_buffer() {return 0;}
+extern void* wave_test_get_write_buffer() {return NULL;}
+
+int wave_get_remaining_time(uint32_t sample, uint32_t* time)
+{
+	if (!time) return(-1);
+	*time = (uint32_t)0;
+	return 0;
+}
 
 #endif  // of USE_PORTAUDIO
 
