@@ -1,10 +1,10 @@
 /***************************************************************************
- *   Copyright (C) 2005, 2006 by Jonathan Duddington                       *
- *   jonsd@users.sourceforge.net                                           *
+ *   Copyright (C) 2005 to 2007 by Jonathan Duddington                     *
+ *   email: jonsd@users.sourceforge.net                                    *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
- *   the Free Software Foundation; either version 2 of the License, or     *
+ *   the Free Software Foundation; either version 3 of the License, or     *
  *   (at your option) any later version.                                   *
  *                                                                         *
  *   This program is distributed in the hope that it will be useful,       *
@@ -13,10 +13,10 @@
  *   GNU General Public License for more details.                          *
  *                                                                         *
  *   You should have received a copy of the GNU General Public License     *
- *   along with this program; if not, write to the                         *
- *   Free Software Foundation, Inc.,                                       *
- *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
+ *   along with this program; if not, see:                                 *
+ *               <http://www.gnu.org/licenses/>.                           *
  ***************************************************************************/
+
 #include "StdAfx.h"
 
 // this version keeps wavemult window as a constant fraction
@@ -30,9 +30,9 @@
 
 #include "speak_lib.h"
 #include "speech.h"
-#include "voice.h"
 #include "phoneme.h"
 #include "synthesize.h"
+#include "voice.h"
 
 #ifdef USE_PORTAUDIO
 #include "portaudio.h"
@@ -49,16 +49,17 @@
 #include "sintab.h"
 
 
-
+#define PI  3.1415927
 #define PI2 6.283185307
 #define STEPSIZE  64                // 2.9mS at 22 kHz sample rate
-#define N_WAV_BUF   15
+#define N_WAV_BUF   10
 
 static voice_t *wvoice;
 
 FILE *f_log = NULL;
 int option_waveout = 0;
 int option_harmonic1 = 11;   // 10
+int option_log_frames = 0;
 static int flutter_amp = 64;
 
 static int general_amplitude = 60;
@@ -68,7 +69,9 @@ int embedded_value[N_EMBEDDED_VALUES];
 
 static int PHASE_INC_FACTOR;
 int samplerate = 0;       // this is set by Wavegeninit()
+int samplerate_native=0;
 extern int option_device_number;
+extern int option_quiet;
 
 static wavegen_peaks_t peaks[N_PEAKS];
 static int peak_harmonic[N_PEAKS];
@@ -80,6 +83,9 @@ static int echo_tail;
 static int echo_amp = 0;
 static short echo_buf[N_ECHO_BUF];
 
+static int voicing;
+RESONATOR rbreath[N_PEAKS];
+
 static int harm_sqrt_n = 0;
 
 
@@ -122,9 +128,14 @@ static int cycle_samples;         // number of samples in a cycle at current pit
 static int cbytes;
 static int hf_factor;
 
+static double minus_pi_t;
+static double two_pi_t;
+
 
 unsigned char *out_ptr;
+unsigned char *out_start;
 unsigned char *out_end;
+int outbuf_size = 0;
 static unsigned char outbuf[1024];  // used when writing to file
 
 // the queue of operations passed to wavegen from sythesize
@@ -133,8 +144,8 @@ int wcmdq_head=0;
 int wcmdq_tail=0;
 
 // pitch,speed,
-int embedded_default[N_EMBEDDED_VALUES]        = {0,50,165,100,50, 0,50, 0,165,0,0,0,0,0};
-static int embedded_max[N_EMBEDDED_VALUES]     = {0,99,320,300,99,99,99, 0,320,0,0,0,0,4};
+int embedded_default[N_EMBEDDED_VALUES]        = {0,50,170,100,50, 0,50, 0,170,0,0,0,0,0};
+static int embedded_max[N_EMBEDDED_VALUES]     = {0,0x7fff,360,300,99,99,99, 0,360,0,0,0,0,4};
 
 #define N_CALLBACK_IX N_WAV_BUF-2   // adjust this delay to match display with the currently spoken word
 int current_source_index=0;
@@ -183,14 +194,14 @@ unsigned char Pitch_long[ENV_LEN] = {
 // value: bits 0-3  amplitude (16ths), bits 4-7 every n cycles
 #define N_ROUGHNESS 8
 static unsigned char modulation_tab[N_ROUGHNESS][8] = {
-	{0, 0x00, 0x00, 0x00, 0, 0x35, 0xf2, 0x29},
-	{0, 0x2f, 0x00, 0x2f, 0, 0x34, 0xf2, 0x29},
-	{0, 0x2f, 0x00, 0x2e, 0, 0x33, 0xf2, 0x28},
-	{0, 0x2e, 0x00, 0x2d, 0, 0x33, 0xf2, 0x28},
-	{0, 0x2d, 0x2d, 0x2c, 0, 0x33, 0xf2, 0x28},
-	{0, 0x2b, 0x2b, 0x2b, 0, 0x33, 0xf2, 0x28},
-	{0, 0x2a, 0x2a, 0x2a, 0, 0x33, 0xf2, 0x28},
-	{0, 0x29, 0x29, 0x29, 0, 0x33, 0xf2, 0x28},
+	{0, 0x00, 0x00, 0x00, 0, 0x46, 0xf2, 0x29},
+	{0, 0x2f, 0x00, 0x2f, 0, 0x45, 0xf2, 0x29},
+	{0, 0x2f, 0x00, 0x2e, 0, 0x45, 0xf2, 0x28},
+	{0, 0x2e, 0x00, 0x2d, 0, 0x34, 0xf2, 0x28},
+	{0, 0x2d, 0x2d, 0x2c, 0, 0x34, 0xf2, 0x28},
+	{0, 0x2b, 0x2b, 0x2b, 0, 0x34, 0xf2, 0x28},
+	{0, 0x2a, 0x2a, 0x2a, 0, 0x34, 0xf2, 0x28},
+	{0, 0x29, 0x29, 0x29, 0, 0x34, 0xf2, 0x28},
 };
 
 // Flutter table, to add natural variations to the pitch
@@ -293,7 +304,7 @@ static unsigned char wavemult[N_WAVEMULT] = {
    
 
 // set from y = pow(2,x) * 128,  x=-1 to 1
-unsigned char pitch_adjust_tab[100] = {
+unsigned char pitch_adjust_tab[MAX_PITCH_VALUE+1] = {
     64, 65, 66, 67, 68, 69, 70, 71,
     72, 73, 74, 75, 76, 77, 78, 79,
     80, 81, 82, 83, 84, 86, 87, 88,
@@ -306,11 +317,33 @@ unsigned char pitch_adjust_tab[100] = {
    174,176,179,181,184,186,189,191,
    194,197,199,202,205,208,211,214,
    217,220,223,226,229,232,236,239,
-   242,246,249,252 };
+   242,246,249,252, 254,255 };
 
 int WavegenFill(int fill_zeros);
 
 
+#ifdef LOG_FRAMES
+static void LogMarker(int type, int value)
+{//=======================================
+	if(option_log_frames == 0)
+		return;
+
+	if((type == espeakEVENT_PHONEME) || (type == espeakEVENT_SENTENCE))
+	{
+		f_log=fopen("log-espeakedit","a");
+		if(f_log)
+		{
+			if(type == espeakEVENT_PHONEME)
+				fprintf(f_log,"Phoneme [%s]\n",WordToString(value));
+			else
+				fprintf(f_log,"\n");
+			fclose(f_log);
+			f_log = NULL;
+		}
+	}
+}
+#endif
+
 void WcmdqStop()
 {//=============
 	wcmdq_head = 0;
@@ -454,11 +487,29 @@ static int WaveCallback(const void *inputBuffer, void *outputBuffer,
 	int result;
 	unsigned char *p;
 
-	out_ptr = (unsigned char *)outputBuffer;
+	out_ptr = out_start = (unsigned char *)outputBuffer;
 	out_end = out_ptr + framesPerBuffer*2;
 
+#ifdef LIBRARY
+	event_list_ix = 0;
+#endif
+
 	result = WavegenFill(1);
 
+#ifdef LIBRARY
+	count_samples += framesPerBuffer;
+	if(synth_callback)
+	{
+		// synchronous-playback mode, allow the calling process to abort the speech
+		event_list[event_list_ix].type = espeakEVENT_LIST_TERMINATED; // indicates end of event list
+		if(synth_callback(NULL,0,event_list) == 1)
+		{
+			SpeakNextClause(NULL,NULL,2);  // stop speaking
+			result = 1;
+		}
+	}
+#endif
+
 #ifdef ARCH_BIG
 	{
 		// swap the order of bytes in each sound sample in the portaudio buffer
@@ -489,6 +540,9 @@ static int WaveCallback(const void *inputBuffer, void *outputBuffer,
 	}
 
 #if USE_PORTAUDIO == 18
+#ifdef PLATFORM_WINDOWS
+	return(result);
+#endif
 	if(result != 0)
 	{
 		static int end_timer = 0;
@@ -557,7 +611,7 @@ int WavegenOpenSound()
 	PaError err, err2;
 	PaError active;
 
-	if(option_waveout)
+	if(option_waveout || option_quiet)
 	{
 		// writing to WAV file, not to portaudio
 		return(0);
@@ -621,7 +675,8 @@ int WavegenCloseSound()
 	PaError active;
 
 	// check whether speaking has finished, and close the stream
-	if(pa_stream != NULL)
+//	if(pa_stream != NULL)
+	if((pa_stream != NULL) && (WcmdqUsed() == 0))  // TEST, also check that queue is empty
 	{
 #if USE_PORTAUDIO == 18
 		active = Pa_StreamActive(pa_stream);
@@ -643,6 +698,9 @@ int WavegenInitSound()
 {//===================
 	PaError err;
 
+	if(option_quiet)
+		return(0);
+
 	// PortAudio sound output library
 	err = Pa_Initialize();
 	pa_init_err = err;
@@ -677,7 +735,7 @@ void WavegenInit(int rate, int wavemult_fact)
 	if(wavemult_fact == 0)
 		wavemult_fact=60;  // default
 
-	samplerate = rate;
+	samplerate = samplerate_native = rate;
 	PHASE_INC_FACTOR = 0x8000000 / samplerate;   // assumes pitch is Hz*32
 	Flutter_inc = (64 * samplerate)/rate;
 	samplecount = 0;
@@ -741,6 +799,7 @@ static void WavegenSetEcho(void)
 	int delay;
 	int amp;
 
+	voicing = wvoice->voicing;
 	delay = wvoice->echo_delay;
 	amp = wvoice->echo_amp;
 
@@ -795,6 +854,8 @@ int PeaksToHarmspect(wavegen_peaks_t *peaks, int pitch, int *htab, int control)
 	if(wvoice == NULL)
 		return(1);
 	hmax = (peaks[wvoice->n_harmonic_peaks].freq + peaks[wvoice->n_harmonic_peaks].right)/pitch;
+	if(hmax >= MAX_HARMONIC)
+		hmax = MAX_HARMONIC-1;
 
 	// restrict highest harmonic to half the samplerate
 	hmax_samplerate = (((samplerate * 19)/40) << 16)/pitch;   // only 95% of Nyquist freq
@@ -932,6 +993,109 @@ static void AdvanceParameters()
 
 
 
+static double resonator(RESONATOR *r, double input)
+{//================================================
+	double x;
+
+	x = r->a * input + r->b * r->x1 + r->c * r->x2;
+	r->x2 = r->x1;
+	r->x1 = x;
+
+ return x;
+}
+
+
+
+static void setresonator(RESONATOR *rp, int freq, int bwidth, int init)
+{//====================================================================
+// freq    Frequency of resonator in Hz
+// bwidth  Bandwidth of resonator in Hz
+// init    Initialize internal data
+
+	double x;
+	double arg;
+
+	if(init)
+	{
+		rp->x1 = 0;
+		rp->x2 = 0;
+	}
+
+   // x  =  exp(-pi * bwidth * t)
+	arg = minus_pi_t * bwidth;
+	x = exp(arg);
+
+	// c  =  -(x*x)
+	rp->c = -(x * x);
+
+	// b = x * 2*cos(2 pi * freq * t)
+
+	arg = two_pi_t * freq;
+	rp->b = x * cos(arg) * 2.0;
+
+	// a = 1.0 - b - c
+	rp->a = 1.0 - rp->b - rp->c;
+}  // end if setresonator
+
+
+
+void InitBreath(void)
+{//==================
+	int ix;
+
+	minus_pi_t = -PI / samplerate;
+	two_pi_t = -2.0 * minus_pi_t;
+
+	for(ix=0; ix<N_PEAKS; ix++)
+	{
+		setresonator(&rbreath[ix],2000,200,1);
+	}
+}  // end of InitBreath
+
+
+
+void SetBreath()
+{//=============
+	int pk;
+
+	if(wvoice->breath[0] == 0)
+		return;
+
+	for(pk=1; pk<N_PEAKS; pk++)
+	{
+		if(wvoice->breath[pk] != 0)
+		{
+			// breath[0] indicates that some breath formants are needed
+			// set the freq from the current ynthesis formant and the width from the voice data
+			setresonator(&rbreath[pk], peaks[pk].freq >> 16, wvoice->breathw[pk],0);
+		}
+	}
+}  // end of SetBreath
+
+
+int ApplyBreath(void)
+{//==================
+	int noise;
+	int ix;
+	int amp;
+	int value = 0;
+
+	// use two random numbers, for alternate formants
+	noise = (rand() & 0x3fff) - 0x2000;
+
+	for(ix=1; ix < N_PEAKS; ix++)
+	{
+		if((amp = wvoice->breath[ix]) != 0)
+		{
+			amp *= (peaks[ix].height >> 14);
+			value += int(resonator(&rbreath[ix],noise) * amp);
+		}
+	}
+	return (value);
+}
+
+
+
 static int Wavegen()
 {//=================
 	unsigned short waveph;
@@ -968,6 +1132,8 @@ static int Wavegen()
 				hswitch = 0;
 				harmspect = hspect[0];
 				maxh2 = PeaksToHarmspect(peaks,pitch<<4,hspect[0],0);
+
+				// adjust amplitude to compensate for fewer harmonics at higher pitch
 				amplitude2 = (amplitude * pitch)/(100 << 12);
 
             // switch sign of harmonics above about 900Hz, to reduce max peak amplitude
@@ -986,6 +1152,7 @@ static int Wavegen()
 			hswitch ^= 1;
 			maxh2 = PeaksToHarmspect(peaks,pitch<<4,hspect[hswitch],1);
 
+			SetBreath();
 		}
 		else
 		if((samplecount & 0x07) == 0)
@@ -1121,6 +1288,17 @@ static int Wavegen()
 			h++;
 		}
 #endif
+
+		if(voicing != 64)
+		{
+			total = (total >> 6) * voicing;
+		}
+
+		if(wvoice->breath[0])
+		{
+			total +=  ApplyBreath();
+		}
+
 		// mix with sampled wave if required
 		z2 = 0;
 		if(mix_wavefile_ix < n_mix_wavefile)
@@ -1286,6 +1464,7 @@ void SetEmbedded(int control, int value)
 	int command;
 	int ix;
 	int factor;
+	int pitch_value;
 
 	command = control & 0x1f;
 	if((control & 0x60) == 0x60)
@@ -1308,7 +1487,10 @@ void SetEmbedded(int control, int value)
 	case EMBED_P:
 	case EMBED_T:
 		// adjust formants to give better results for a different voice pitch
-		factor = 256 + (25 * (embedded_value[EMBED_P] - 50))/50;
+		if((pitch_value = embedded_value[EMBED_P]) > MAX_PITCH_VALUE)
+			pitch_value = MAX_PITCH_VALUE;
+
+		factor = 256 + (25 * (pitch_value - 50))/50;
 		for(ix=0; ix<=5; ix++)
 		{
 			wvoice->freq[ix] = (wvoice->freq2[ix] * factor)/256;
@@ -1335,7 +1517,10 @@ void SetEmbedded(int control, int value)
 
 void WavegenSetVoice(voice_t *v)
 {//=============================
-	wvoice = v;
+	static voice_t v2;
+
+	memcpy(&v2,v,sizeof(v2));
+	wvoice = &v2;
 
 	if(v->peak_shape==0)
 		pk_shape = pk_shape1;
@@ -1367,14 +1552,18 @@ void SetPitch(int length, unsigned char *env, int pitch1, int pitch2)
 	int x;
 	int base;
 	int range;
+	int pitch_value;
 
 #ifdef LOG_FRAMES
-f_log=fopen("log-espeakedit","a");
-if(f_log != NULL)
+if(option_log_frames)
 {
-fprintf(f_log,"	  %3d %3d\n",pitch1,pitch2);
-fclose(f_log);
-f_log=NULL;
+	f_log=fopen("log-espeakedit","a");
+	if(f_log != NULL)
+	{
+		fprintf(f_log,"	  pitch %3d %3d  %3dmS\n",pitch1,pitch2,(length*1000)/samplerate);
+		fclose(f_log);
+		f_log=NULL;
+	}
 }
 #endif
 	if((pitch_env = env)==NULL)
@@ -1393,7 +1582,12 @@ f_log=NULL;
 		pitch2 = x;
 	}
 
-	base = (wvoice->pitch_base * pitch_adjust_tab[embedded_value[EMBED_P]])/128;
+	if((pitch_value = embedded_value[EMBED_P]) > MAX_PITCH_VALUE)
+		pitch_value = MAX_PITCH_VALUE;
+	if(pitch_value < 0)
+		pitch_value = 0;
+
+	base = (wvoice->pitch_base * pitch_adjust_tab[pitch_value])/128;
 	range =  (wvoice->pitch_range * embedded_value[EMBED_R])/50;
 
 	// compensate for change in pitch when the range is narrowed or widened
@@ -1427,14 +1621,18 @@ void SetSynth(int length, int modn, frame_t *fr1, frame_t *fr2)
 	static int glottal_reduce_tab2[4] = {0x90, 0xa0, 0xb0, 0xc0};  // vowel after [?], amp * 1/256
 
 #ifdef LOG_FRAMES
-f_log=fopen("log-espeakedit","a");
-if(f_log != NULL)
+if(option_log_frames)
 {
-fprintf(f_log,"%3dmS  %4d/%3d %4d/%3d %4d/%3d %4d/%3d   %4d/%3d %4d/%3d %4d/%3d %4d/%3d\n",length*1000/samplerate,
-	fr1->ffreq[0],fr1->fheight[0],fr1->ffreq[1],fr1->fheight[1], fr1->ffreq[2],fr1->fheight[2], fr1->ffreq[3],fr1->fheight[3],
-	fr2->ffreq[0],fr2->fheight[0],fr2->ffreq[1],fr2->fheight[1], fr2->ffreq[2],fr2->fheight[2], fr2->ffreq[3],fr2->fheight[3] );
-fclose(f_log);
-f_log=NULL;
+	f_log=fopen("log-espeakedit","a");
+	if(f_log != NULL)
+	{
+		fprintf(f_log,"%3dmS  %3d %3d %4d %4d (%3d %3d %3d %3d)  to  %3d %3d %4d %4d (%3d %3d %3d %3d)\n",length*1000/samplerate,
+			fr1->ffreq[0],fr1->ffreq[1],fr1->ffreq[2],fr1->ffreq[3], fr1->fheight[0],fr1->fheight[1],fr1->fheight[2],fr1->fheight[3],
+			fr2->ffreq[0],fr2->ffreq[1],fr2->ffreq[2],fr2->ffreq[3], fr2->fheight[0],fr2->fheight[1],fr2->fheight[2],fr2->fheight[3] );
+	
+	fclose(f_log);
+	f_log=NULL;
+	}
 }
 #endif
 
@@ -1603,7 +1801,7 @@ void MakeWaveFile()
 
 	while(result != 0)
 	{
-		out_ptr = outbuf;
+		out_ptr = out_start = outbuf;
 		out_end = &outbuf[sizeof(outbuf)];
 		result = Wavegen();
 		if(f_wave != NULL)
@@ -1625,8 +1823,11 @@ int WavegenFill(int fill_zeros)
 	int length;
 	int result;
 	static int resume=0;
-//	static unsigned char speed_adjust_tab[10] = {223,199,179,160,143,128,115,103,92,82};
 
+#ifdef TEST_MBROLA
+	if(mbrola_name[0] != 0)
+		return(MbrolaFill(fill_zeros));
+#endif
 
 	while(out_ptr < out_end)
 	{
@@ -1681,7 +1882,9 @@ int WavegenFill(int fill_zeros)
 
 		case WCMD_MARKER:
 			MarkerEvent(q[1],q[2],q[3],out_ptr);
-
+#ifdef LOG_FRAMES
+			LogMarker(q[1],q[3]);
+#endif
 			if(q[1] == 1)
 			{
 				current_source_index = q[2] & 0xffffff;
@@ -1694,6 +1897,7 @@ int WavegenFill(int fill_zeros)
 
 		case WCMD_VOICE:
 			WavegenSetVoice((voice_t *)q[1]);
+			free((voice_t *)q[1]);
 			break;
 
 		case WCMD_EMBEDDED:
@@ -1721,7 +1925,7 @@ int WavegenFile(void)
 {//==================
 	int finished;
 
-	out_ptr = outbuf;
+	out_ptr = out_start = outbuf;
 	out_end = outbuf + sizeof(outbuf);
 
 	finished = WavegenFill(0);
